# This file is generated

from typing import Any

from ..structure import to_nullable_object_schema, array_schema

COMMENT_OPERATIONS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "view": {
            "type": ["string", "null"]
        },
        "edit": {
            "type": ["string", "null"]
        },
        "delete": {
            "type": ["string", "null"]
        },
        "viewReactions": {
            "type": ["string", "null"]
        },
        "viewNegativeReactions": {
            "type": ["string", "null"]
        },
        "viewReactionTotals": {
            "type": ["string", "null"]
        },
        "viewNegativeReactionTotals": {
            "type": ["string", "null"]
        },
        "viewReactionRatios": {
            "type": ["string", "null"]
        },
        "viewNegativeReactionRatios": {
            "type": ["string", "null"]
        },
        "addReaction": {
            "type": ["string", "null"]
        },
        "addNegativeReaction": {
            "type": ["string", "null"]
        },
        "overrideReaction": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

CONTACT_OPERATIONS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "viewFeedSubscriber": {
            "type": ["string", "null"]
        },
        "viewFeedSubscription": {
            "type": ["string", "null"]
        },
        "viewFriend": {
            "type": ["string", "null"]
        },
        "viewFriendOf": {
            "type": ["string", "null"]
        },
        "viewBlock": {
            "type": ["string", "null"]
        },
        "viewBlockBy": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

FEED_OPERATIONS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "add": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

FRIEND_OPERATIONS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "view": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

FRIEND_GROUP_OPERATIONS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "view": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

NODE_NAME_OPERATIONS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "manage": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

PEOPLE_OPERATIONS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "viewSubscribers": {
            "type": ["string", "null"]
        },
        "viewSubscriptions": {
            "type": ["string", "null"]
        },
        "viewFriends": {
            "type": ["string", "null"]
        },
        "viewFriendOfs": {
            "type": ["string", "null"]
        },
        "viewBlocked": {
            "type": ["string", "null"]
        },
        "viewBlockedBy": {
            "type": ["string", "null"]
        },
        "viewSubscribersTotal": {
            "type": ["string", "null"]
        },
        "viewSubscriptionsTotal": {
            "type": ["string", "null"]
        },
        "viewFriendsTotal": {
            "type": ["string", "null"]
        },
        "viewFriendOfsTotal": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

POSTING_OPERATIONS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "view": {
            "type": ["string", "null"]
        },
        "edit": {
            "type": ["string", "null"]
        },
        "delete": {
            "type": ["string", "null"]
        },
        "viewComments": {
            "type": ["string", "null"]
        },
        "addComment": {
            "type": ["string", "null"]
        },
        "overrideComment": {
            "type": ["string", "null"]
        },
        "viewReactions": {
            "type": ["string", "null"]
        },
        "viewNegativeReactions": {
            "type": ["string", "null"]
        },
        "viewReactionTotals": {
            "type": ["string", "null"]
        },
        "viewNegativeReactionTotals": {
            "type": ["string", "null"]
        },
        "viewReactionRatios": {
            "type": ["string", "null"]
        },
        "viewNegativeReactionRatios": {
            "type": ["string", "null"]
        },
        "addReaction": {
            "type": ["string", "null"]
        },
        "addNegativeReaction": {
            "type": ["string", "null"]
        },
        "overrideReaction": {
            "type": ["string", "null"]
        },
        "overrideCommentReaction": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

PRIVATE_MEDIA_FILE_OPERATIONS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "view": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

PROFILE_OPERATIONS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "edit": {
            "type": ["string", "null"]
        },
        "viewEmail": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

REACTION_OPERATIONS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "view": {
            "type": ["string", "null"]
        },
        "delete": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

SEARCH_ENTRY_OPERATIONS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "view": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

STORY_OPERATIONS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "edit": {
            "type": ["string", "null"]
        },
        "delete": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

SUBSCRIBER_OPERATIONS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "view": {
            "type": ["string", "null"]
        },
        "delete": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

SUBSCRIPTION_OPERATIONS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "view": {
            "type": ["string", "null"]
        },
        "delete": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

ASYNC_OPERATION_CREATED_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
    },
    "required": [
        "id",
    ],
    "additionalProperties": False
}

AVATAR_IMAGE_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "mediaId": {
            "type": "string"
        },
        "path": {
            "type": "string"
        },
        "width": {
            "type": ["integer", "null"]
        },
        "height": {
            "type": ["integer", "null"]
        },
        "shape": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "mediaId",
        "path",
    ],
    "additionalProperties": False
}

AVATAR_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "mediaId": {
            "type": "string"
        },
        "path": {
            "type": "string"
        },
        "width": {
            "type": ["integer", "null"]
        },
        "height": {
            "type": ["integer", "null"]
        },
        "shape": {
            "type": ["string", "null"]
        },
        "ordinal": {
            "type": "integer"
        },
    },
    "required": [
        "id",
        "mediaId",
        "path",
        "ordinal",
    ],
    "additionalProperties": False
}

AVATAR_INFO_ARRAY_SCHEMA = array_schema(AVATAR_INFO_SCHEMA)

AVATAR_ORDINAL_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "ordinal": {
            "type": "integer"
        },
    },
    "required": [
        "id",
        "ordinal",
    ],
    "additionalProperties": False
}

AVATAR_ORDINAL_ARRAY_SCHEMA = array_schema(AVATAR_ORDINAL_SCHEMA)

BLOCKED_INSTANT_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "storyType": {
            "type": "string"
        },
        "entryId": {
            "type": ["string", "null"]
        },
        "remoteNodeName": {
            "type": ["string", "null"]
        },
        "remotePostingId": {
            "type": ["string", "null"]
        },
        "remoteOwnerName": {
            "type": ["string", "null"]
        },
        "createdAt": {
            "type": "integer"
        },
        "deadline": {
            "type": ["integer", "null"]
        },
    },
    "required": [
        "id",
        "storyType",
        "createdAt",
    ],
    "additionalProperties": False
}

BLOCKED_INSTANT_INFO_ARRAY_SCHEMA = array_schema(BLOCKED_INSTANT_INFO_SCHEMA)

BLOCKED_POSTING_INSTANT_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "storyType": {
            "type": "string"
        },
        "remoteOwnerName": {
            "type": ["string", "null"]
        },
        "deadline": {
            "type": ["integer", "null"]
        },
    },
    "required": [
        "id",
        "storyType",
    ],
    "additionalProperties": False
}

BLOCKED_USERS_CHECKSUMS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "visibility": {
            "type": "integer"
        },
    },
    "required": [
        "visibility",
    ],
    "additionalProperties": False
}

CARTE_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "carte": {
            "type": "string"
        },
        "beginning": {
            "type": "integer"
        },
        "deadline": {
            "type": "integer"
        },
        "nodeName": {
            "type": ["string", "null"]
        },
        "clientScope": {
            "type": ["array", "null"],
            "items": {
                "type": "string"
            }
        },
        "adminScope": {
            "type": ["array", "null"],
            "items": {
                "type": "string"
            }
        },
    },
    "required": [
        "carte",
        "beginning",
        "deadline",
    ],
    "additionalProperties": False
}

CARTE_SET_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "cartesIp": {
            "type": ["string", "null"]
        },
        "cartes": {
            "type": "array",
            "items": CARTE_INFO_SCHEMA
        },
        "createdAt": {
            "type": "integer"
        },
    },
    "required": [
        "cartes",
        "createdAt",
    ],
    "additionalProperties": False
}

CARTE_VERIFICATION_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "valid": {
            "type": "boolean"
        },
        "clientName": {
            "type": ["string", "null"]
        },
        "clientScope": {
            "type": ["array", "null"],
            "items": {
                "type": "string"
            }
        },
        "adminScope": {
            "type": ["array", "null"],
            "items": {
                "type": "string"
            }
        },
        "errorCode": {
            "type": ["string", "null"]
        },
        "errorMessage": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "valid",
    ],
    "additionalProperties": False
}

CLIENT_REACTION_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "negative": {
            "type": "boolean"
        },
        "emoji": {
            "type": "integer"
        },
        "createdAt": {
            "type": "integer"
        },
        "deadline": {
            "type": ["integer", "null"]
        },
    },
    "required": [
        "negative",
        "emoji",
        "createdAt",
    ],
    "additionalProperties": False
}

COMMENT_TOTAL_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "total": {
            "type": "integer"
        },
    },
    "required": [
        "total",
    ],
    "additionalProperties": False
}

CONTACT_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "nodeName": {
            "type": "string"
        },
        "fullName": {
            "type": ["string", "null"]
        },
        "gender": {
            "type": ["string", "null"]
        },
        "avatar": to_nullable_object_schema(AVATAR_IMAGE_SCHEMA),
        "distance": {
            "type": "number"
        },
        "hasFeedSubscriber": {
            "type": ["boolean", "null"]
        },
        "hasFeedSubscription": {
            "type": ["boolean", "null"]
        },
        "hasFriend": {
            "type": ["boolean", "null"]
        },
        "hasFriendOf": {
            "type": ["boolean", "null"]
        },
        "hasBlock": {
            "type": ["boolean", "null"]
        },
        "hasBlockBy": {
            "type": ["boolean", "null"]
        },
        "operations": to_nullable_object_schema(CONTACT_OPERATIONS_SCHEMA),
        "ownerOperations": to_nullable_object_schema(CONTACT_OPERATIONS_SCHEMA),
        "adminOperations": to_nullable_object_schema(CONTACT_OPERATIONS_SCHEMA),
    },
    "required": [
        "nodeName",
        "distance",
    ],
    "additionalProperties": False
}

CONTACT_INFO_ARRAY_SCHEMA = array_schema(CONTACT_INFO_SCHEMA)

CREDENTIALS_CREATED_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "created": {
            "type": "boolean"
        },
    },
    "required": [
        "created",
    ],
    "additionalProperties": False
}

DELETE_NODE_STATUS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "requested": {
            "type": "boolean"
        },
    },
    "required": [
        "requested",
    ],
    "additionalProperties": False
}

DOMAIN_AVAILABLE_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "name": {
            "type": "string"
        },
    },
    "required": [
        "name",
    ],
    "additionalProperties": False
}

DOMAIN_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "name": {
            "type": "string"
        },
        "nodeId": {
            "type": "string"
        },
        "createdAt": {
            "type": "integer"
        },
    },
    "required": [
        "name",
        "nodeId",
        "createdAt",
    ],
    "additionalProperties": False
}

DOMAIN_INFO_ARRAY_SCHEMA = array_schema(DOMAIN_INFO_SCHEMA)

EMAIL_HINT_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "emailHint": {
            "type": "string"
        },
    },
    "required": [
        "emailHint",
    ],
    "additionalProperties": False
}

FEED_REFERENCE_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "feedName": {
            "type": "string"
        },
        "publishedAt": {
            "type": "integer"
        },
        "pinned": {
            "type": ["boolean", "null"]
        },
        "moment": {
            "type": "integer"
        },
        "storyId": {
            "type": "string"
        },
        "operations": to_nullable_object_schema(STORY_OPERATIONS_SCHEMA),
    },
    "required": [
        "feedName",
        "publishedAt",
        "moment",
        "storyId",
    ],
    "additionalProperties": False
}

FEED_STATUS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "total": {
            "type": "integer"
        },
        "totalPinned": {
            "type": "integer"
        },
        "lastMoment": {
            "type": ["integer", "null"]
        },
        "notViewed": {
            "type": ["integer", "null"]
        },
        "notRead": {
            "type": ["integer", "null"]
        },
        "notViewedMoment": {
            "type": ["integer", "null"]
        },
        "notReadMoment": {
            "type": ["integer", "null"]
        },
    },
    "required": [
        "total",
        "totalPinned",
    ],
    "additionalProperties": False
}

FEED_WITH_STATUS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "feedName": {
            "type": "string"
        },
        "notViewed": {
            "type": "integer"
        },
        "notRead": {
            "type": "integer"
        },
        "notViewedMoment": {
            "type": ["integer", "null"]
        },
        "notReadMoment": {
            "type": ["integer", "null"]
        },
    },
    "required": [
        "feedName",
        "notViewed",
        "notRead",
    ],
    "additionalProperties": False
}

FRIEND_GROUP_DETAILS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "title": {
            "type": ["string", "null"]
        },
        "addedAt": {
            "type": "integer"
        },
        "operations": to_nullable_object_schema(FRIEND_OPERATIONS_SCHEMA),
    },
    "required": [
        "id",
        "addedAt",
    ],
    "additionalProperties": False
}

FRIEND_GROUP_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "title": {
            "type": ["string", "null"]
        },
        "createdAt": {
            "type": "integer"
        },
        "operations": to_nullable_object_schema(FRIEND_GROUP_OPERATIONS_SCHEMA),
    },
    "required": [
        "id",
        "createdAt",
    ],
    "additionalProperties": False
}

FRIEND_GROUP_INFO_ARRAY_SCHEMA = array_schema(FRIEND_GROUP_INFO_SCHEMA)

FRIEND_GROUPS_FEATURES_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "available": {
            "type": "array",
            "items": FRIEND_GROUP_INFO_SCHEMA
        },
        "memberOf": {
            "type": ["array", "null"],
            "items": FRIEND_GROUP_DETAILS_SCHEMA
        },
    },
    "required": [
        "available",
    ],
    "additionalProperties": False
}

FRIEND_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "nodeName": {
            "type": "string"
        },
        "contact": to_nullable_object_schema(CONTACT_INFO_SCHEMA),
        "groups": {
            "type": ["array", "null"],
            "items": FRIEND_GROUP_DETAILS_SCHEMA
        },
    },
    "required": [
        "nodeName",
    ],
    "additionalProperties": False
}

FRIEND_INFO_ARRAY_SCHEMA = array_schema(FRIEND_INFO_SCHEMA)

FRIEND_OF_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "remoteNodeName": {
            "type": "string"
        },
        "contact": to_nullable_object_schema(CONTACT_INFO_SCHEMA),
        "groups": {
            "type": ["array", "null"],
            "items": FRIEND_GROUP_DETAILS_SCHEMA
        },
    },
    "required": [
        "remoteNodeName",
    ],
    "additionalProperties": False
}

FRIEND_OF_INFO_ARRAY_SCHEMA = array_schema(FRIEND_OF_INFO_SCHEMA)

FUNDRAISER_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "title": {
            "type": "string"
        },
        "qrCode": {
            "type": ["string", "null"]
        },
        "text": {
            "type": ["string", "null"]
        },
        "href": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "title",
    ],
    "additionalProperties": False
}

GRANT_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "nodeName": {
            "type": "string"
        },
        "scope": {
            "type": "array",
            "items": {
                "type": "string"
            }
        },
    },
    "required": [
        "nodeName",
        "scope",
    ],
    "additionalProperties": False
}

GRANT_INFO_ARRAY_SCHEMA = array_schema(GRANT_INFO_SCHEMA)

KEY_MNEMONIC_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "mnemonic": {
            "type": "array",
            "items": {
                "type": "string"
            }
        },
    },
    "required": [
        "mnemonic",
    ],
    "additionalProperties": False
}

LINK_PREVIEW_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "siteName": {
            "type": ["string", "null"]
        },
        "url": {
            "type": ["string", "null"]
        },
        "title": {
            "type": ["string", "null"]
        },
        "description": {
            "type": ["string", "null"]
        },
        "imageHash": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

LINK_PREVIEW_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "siteName": {
            "type": ["string", "null"]
        },
        "url": {
            "type": ["string", "null"]
        },
        "title": {
            "type": ["string", "null"]
        },
        "description": {
            "type": ["string", "null"]
        },
        "imageUrl": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

MEDIA_FILE_PREVIEW_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "targetWidth": {
            "type": "integer"
        },
        "directPath": {
            "type": ["string", "null"]
        },
        "width": {
            "type": "integer"
        },
        "height": {
            "type": "integer"
        },
        "original": {
            "type": ["boolean", "null"]
        },
    },
    "required": [
        "targetWidth",
        "width",
        "height",
    ],
    "additionalProperties": False
}

NODE_NAME_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "name": {
            "type": ["string", "null"]
        },
        "operationStatus": {
            "type": ["string", "null"]
        },
        "operationStatusUpdated": {
            "type": ["integer", "null"]
        },
        "operationErrorCode": {
            "type": ["string", "null"]
        },
        "operationErrorMessage": {
            "type": ["string", "null"]
        },
        "storedMnemonic": {
            "type": ["boolean", "null"]
        },
        "operations": to_nullable_object_schema(NODE_NAME_OPERATIONS_SCHEMA),
    },
    "additionalProperties": False
}

PEOPLE_GENERAL_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "feedSubscribersTotal": {
            "type": ["integer", "null"]
        },
        "feedSubscriptionsTotal": {
            "type": ["integer", "null"]
        },
        "friendsTotal": {
            "type": ["object", "null"],
            "patternProperties": {
                "^.*$": {
                    "type": "integer"
                }
            }
        },
        "friendOfsTotal": {
            "type": ["integer", "null"]
        },
        "blockedTotal": {
            "type": ["integer", "null"]
        },
        "blockedByTotal": {
            "type": ["integer", "null"]
        },
        "operations": to_nullable_object_schema(PEOPLE_OPERATIONS_SCHEMA),
    },
    "additionalProperties": False
}

POSTING_FEATURES_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "post": {
            "type": ["boolean", "null"]
        },
        "subjectPresent": {
            "type": "boolean"
        },
        "sourceFormats": {
            "type": "array",
            "items": {
                "type": "string"
            }
        },
        "mediaMaxSize": {
            "type": "integer"
        },
        "imageRecommendedSize": {
            "type": "integer"
        },
        "imageRecommendedPixels": {
            "type": "integer"
        },
        "imageFormats": {
            "type": "array",
            "items": {
                "type": "string"
            }
        },
    },
    "required": [
        "subjectPresent",
        "sourceFormats",
        "mediaMaxSize",
        "imageRecommendedSize",
        "imageRecommendedPixels",
        "imageFormats",
    ],
    "additionalProperties": False
}

POSTING_SOURCE_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "nodeName": {
            "type": "string"
        },
        "fullName": {
            "type": ["string", "null"]
        },
        "avatar": to_nullable_object_schema(AVATAR_IMAGE_SCHEMA),
        "feedName": {
            "type": "string"
        },
        "postingId": {
            "type": "string"
        },
        "createdAt": {
            "type": "integer"
        },
    },
    "required": [
        "nodeName",
        "feedName",
        "postingId",
        "createdAt",
    ],
    "additionalProperties": False
}

PRIVATE_MEDIA_FILE_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "hash": {
            "type": "string"
        },
        "path": {
            "type": "string"
        },
        "directPath": {
            "type": ["string", "null"]
        },
        "mimeType": {
            "type": "string"
        },
        "width": {
            "type": ["integer", "null"]
        },
        "height": {
            "type": ["integer", "null"]
        },
        "orientation": {
            "type": ["integer", "null"]
        },
        "size": {
            "type": "integer"
        },
        "textContent": {
            "type": ["string", "null"]
        },
        "postingId": {
            "type": ["string", "null"]
        },
        "previews": {
            "type": ["array", "null"],
            "items": MEDIA_FILE_PREVIEW_INFO_SCHEMA
        },
        "operations": to_nullable_object_schema(PRIVATE_MEDIA_FILE_OPERATIONS_SCHEMA),
    },
    "required": [
        "id",
        "hash",
        "path",
        "mimeType",
        "size",
    ],
    "additionalProperties": False
}

PROFILE_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "fullName": {
            "type": ["string", "null"]
        },
        "gender": {
            "type": ["string", "null"]
        },
        "email": {
            "type": ["string", "null"]
        },
        "emailVerified": {
            "type": ["boolean", "null"]
        },
        "title": {
            "type": ["string", "null"]
        },
        "bioSrc": {
            "type": ["string", "null"]
        },
        "bioSrcFormat": {
            "type": ["string", "null"]
        },
        "bioHtml": {
            "type": ["string", "null"]
        },
        "avatar": to_nullable_object_schema(AVATAR_INFO_SCHEMA),
        "fundraisers": {
            "type": ["array", "null"],
            "items": FUNDRAISER_INFO_SCHEMA
        },
        "operations": to_nullable_object_schema(PROFILE_OPERATIONS_SCHEMA),
    },
    "additionalProperties": False
}

PUBLIC_MEDIA_FILE_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "path": {
            "type": "string"
        },
        "width": {
            "type": ["integer", "null"]
        },
        "height": {
            "type": ["integer", "null"]
        },
        "orientation": {
            "type": ["integer", "null"]
        },
        "size": {
            "type": "integer"
        },
    },
    "required": [
        "id",
        "path",
        "size",
    ],
    "additionalProperties": False
}

REACTION_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "ownerName": {
            "type": ["string", "null"]
        },
        "ownerFullName": {
            "type": ["string", "null"]
        },
        "ownerGender": {
            "type": ["string", "null"]
        },
        "ownerAvatar": to_nullable_object_schema(AVATAR_IMAGE_SCHEMA),
        "postingId": {
            "type": ["string", "null"]
        },
        "postingRevisionId": {
            "type": ["string", "null"]
        },
        "commentId": {
            "type": ["string", "null"]
        },
        "commentRevisionId": {
            "type": ["string", "null"]
        },
        "negative": {
            "type": ["boolean", "null"]
        },
        "emoji": {
            "type": ["integer", "null"]
        },
        "moment": {
            "type": ["integer", "null"]
        },
        "createdAt": {
            "type": ["integer", "null"]
        },
        "deadline": {
            "type": ["integer", "null"]
        },
        "signature": {
            "type": ["string", "null"]
        },
        "signatureVersion": {
            "type": ["integer", "null"]
        },
        "operations": to_nullable_object_schema(REACTION_OPERATIONS_SCHEMA),
        "ownerOperations": to_nullable_object_schema(REACTION_OPERATIONS_SCHEMA),
        "seniorOperations": to_nullable_object_schema(REACTION_OPERATIONS_SCHEMA),
        "majorOperations": to_nullable_object_schema(REACTION_OPERATIONS_SCHEMA),
    },
    "additionalProperties": False
}

REACTION_INFO_ARRAY_SCHEMA = array_schema(REACTION_INFO_SCHEMA)

REACTIONS_SLICE_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "before": {
            "type": "integer"
        },
        "after": {
            "type": "integer"
        },
        "total": {
            "type": "integer"
        },
        "reactions": {
            "type": "array",
            "items": REACTION_INFO_SCHEMA
        },
    },
    "required": [
        "before",
        "after",
        "total",
        "reactions",
    ],
    "additionalProperties": False
}

REACTION_TOTAL_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "emoji": {
            "type": "integer"
        },
        "total": {
            "type": ["integer", "null"]
        },
        "share": {
            "type": ["number", "null"],
            "minimum": 0,
            "maximum": 1
        },
    },
    "required": [
        "emoji",
    ],
    "additionalProperties": False
}

REACTION_TOTALS_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "entryId": {
            "type": "string"
        },
        "positive": {
            "type": ["array", "null"],
            "items": REACTION_TOTAL_INFO_SCHEMA
        },
        "negative": {
            "type": ["array", "null"],
            "items": REACTION_TOTAL_INFO_SCHEMA
        },
    },
    "required": [
        "entryId",
    ],
    "additionalProperties": False
}

REACTION_TOTALS_INFO_ARRAY_SCHEMA = array_schema(REACTION_TOTALS_INFO_SCHEMA)

RECOMMENDED_POSTING_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "nodeName": {
            "type": "string"
        },
        "postingId": {
            "type": "string"
        },
        "ownerName": {
            "type": "string"
        },
        "ownerFullName": {
            "type": ["string", "null"]
        },
        "ownerAvatar": to_nullable_object_schema(AVATAR_IMAGE_SCHEMA),
        "heading": {
            "type": "string"
        },
        "totalPositiveReactions": {
            "type": "integer"
        },
        "lastDayPositiveReactions": {
            "type": "integer"
        },
        "totalComments": {
            "type": "integer"
        },
        "lastDayComments": {
            "type": "integer"
        },
    },
    "required": [
        "nodeName",
        "postingId",
        "ownerName",
        "heading",
        "totalPositiveReactions",
        "lastDayPositiveReactions",
        "totalComments",
        "lastDayComments",
    ],
    "additionalProperties": False
}

RECOMMENDED_POSTING_INFO_ARRAY_SCHEMA = array_schema(RECOMMENDED_POSTING_INFO_SCHEMA)

REGISTERED_NAME_SECRET_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "name": {
            "type": "string"
        },
        "mnemonic": {
            "type": ["array", "null"],
            "items": {
                "type": "string"
            },
            "minItems": 24,
            "maxItems": 24
        },
        "secret": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "name",
    ],
    "additionalProperties": False
}

REJECTED_REACTIONS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "positive": {
            "type": ["string", "null"]
        },
        "negative": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

RECOMMENDED_NODE_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "nodeName": {
            "type": "string"
        },
        "fullName": {
            "type": ["string", "null"]
        },
        "title": {
            "type": ["string", "null"]
        },
        "avatar": to_nullable_object_schema(AVATAR_IMAGE_SCHEMA),
        "subscribersTotal": {
            "type": "integer"
        },
        "postingsTotal": {
            "type": "integer"
        },
    },
    "required": [
        "nodeName",
        "subscribersTotal",
        "postingsTotal",
    ],
    "additionalProperties": False
}

RECOMMENDED_NODE_INFO_ARRAY_SCHEMA = array_schema(RECOMMENDED_NODE_INFO_SCHEMA)

REMOTE_MEDIA_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "hash": {
            "type": ["string", "null"]
        },
        "digest": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "id",
    ],
    "additionalProperties": False
}

REMOTE_POSTING_VERIFICATION_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "nodeName": {
            "type": "string"
        },
        "postingId": {
            "type": "string"
        },
        "revisionId": {
            "type": ["string", "null"]
        },
        "status": {
            "type": ["string", "null"]
        },
        "errorCode": {
            "type": ["string", "null"]
        },
        "errorMessage": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "id",
        "nodeName",
        "postingId",
    ],
    "additionalProperties": False
}

REMOTE_REACTION_VERIFICATION_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "nodeName": {
            "type": "string"
        },
        "postingId": {
            "type": "string"
        },
        "reactionOwnerName": {
            "type": "string"
        },
        "status": {
            "type": ["string", "null"]
        },
        "errorCode": {
            "type": ["string", "null"]
        },
        "errorMessage": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "id",
        "nodeName",
        "postingId",
        "reactionOwnerName",
    ],
    "additionalProperties": False
}

REPLIED_TO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "revisionId": {
            "type": ["string", "null"]
        },
        "name": {
            "type": "string"
        },
        "fullName": {
            "type": ["string", "null"]
        },
        "gender": {
            "type": ["string", "null"]
        },
        "avatar": to_nullable_object_schema(AVATAR_IMAGE_SCHEMA),
        "heading": {
            "type": ["string", "null"]
        },
        "digest": {
            "type": "string"
        },
    },
    "required": [
        "id",
        "name",
        "digest",
    ],
    "additionalProperties": False
}

RESULT_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "errorCode": {
            "type": "string"
        },
        "message": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "errorCode",
    ],
    "additionalProperties": False
}

SHERIFF_MARK_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "sheriffName": {
            "type": "string"
        },
    },
    "required": [
        "sheriffName",
    ],
    "additionalProperties": False
}

SEARCH_HISTORY_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "query": {
            "type": "string"
        },
        "createdAt": {
            "type": "integer"
        },
    },
    "required": [
        "query",
        "createdAt",
    ],
    "additionalProperties": False
}

SEARCH_HISTORY_INFO_ARRAY_SCHEMA = array_schema(SEARCH_HISTORY_INFO_SCHEMA)

SEARCH_NODE_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "nodeName": {
            "type": "string"
        },
        "fullName": {
            "type": ["string", "null"]
        },
        "title": {
            "type": ["string", "null"]
        },
        "avatar": to_nullable_object_schema(AVATAR_IMAGE_SCHEMA),
        "distance": {
            "type": "number"
        },
    },
    "required": [
        "nodeName",
        "distance",
    ],
    "additionalProperties": False
}

SEARCH_NODE_INFO_ARRAY_SCHEMA = array_schema(SEARCH_NODE_INFO_SCHEMA)

SEARCH_NODE_PAGE_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "page": {
            "type": "integer"
        },
        "total": {
            "type": "integer"
        },
        "nodes": {
            "type": "array",
            "items": SEARCH_NODE_INFO_SCHEMA
        },
    },
    "required": [
        "page",
        "total",
        "nodes",
    ],
    "additionalProperties": False
}

SEARCH_REPLIED_TO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "revisionId": {
            "type": ["string", "null"]
        },
        "name": {
            "type": "string"
        },
        "fullName": {
            "type": ["string", "null"]
        },
        "avatar": to_nullable_object_schema(AVATAR_IMAGE_SCHEMA),
        "heading": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "id",
        "name",
    ],
    "additionalProperties": False
}

SETTING_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "name": {
            "type": "string"
        },
        "value": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "name",
    ],
    "additionalProperties": False
}

SETTING_INFO_ARRAY_SCHEMA = array_schema(SETTING_INFO_SCHEMA)

SETTING_VALUE_CHOICE_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "title": {
            "type": "string"
        },
        "value": {
            "type": "string"
        },
    },
    "required": [
        "title",
        "value",
    ],
    "additionalProperties": False
}

SHERIFF_COMPLAINT_GROUP_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "remoteNodeName": {
            "type": "string"
        },
        "remoteNodeFullName": {
            "type": ["string", "null"]
        },
        "remoteFeedName": {
            "type": "string"
        },
        "remotePostingId": {
            "type": ["string", "null"]
        },
        "remotePostingRevisionId": {
            "type": ["string", "null"]
        },
        "remotePostingOwnerName": {
            "type": ["string", "null"]
        },
        "remotePostingOwnerFullName": {
            "type": ["string", "null"]
        },
        "remotePostingOwnerGender": {
            "type": ["string", "null"]
        },
        "remotePostingHeading": {
            "type": ["string", "null"]
        },
        "remoteCommentId": {
            "type": ["string", "null"]
        },
        "remoteCommentRevisionId": {
            "type": ["string", "null"]
        },
        "remoteCommentOwnerName": {
            "type": ["string", "null"]
        },
        "remoteCommentOwnerFullName": {
            "type": ["string", "null"]
        },
        "remoteCommentOwnerGender": {
            "type": ["string", "null"]
        },
        "remoteCommentHeading": {
            "type": ["string", "null"]
        },
        "createdAt": {
            "type": "integer"
        },
        "moment": {
            "type": "integer"
        },
        "status": {
            "type": "string"
        },
        "decisionCode": {
            "type": ["string", "null"]
        },
        "decisionDetails": {
            "type": ["string", "null"]
        },
        "decidedAt": {
            "type": ["integer", "null"]
        },
        "anonymous": {
            "type": ["boolean", "null"]
        },
    },
    "required": [
        "id",
        "remoteNodeName",
        "remoteFeedName",
        "createdAt",
        "moment",
        "status",
    ],
    "additionalProperties": False
}

SHERIFF_COMPLAINT_GROUPS_SLICE_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "before": {
            "type": "integer"
        },
        "after": {
            "type": "integer"
        },
        "groups": {
            "type": "array",
            "items": SHERIFF_COMPLAINT_GROUP_INFO_SCHEMA
        },
        "total": {
            "type": "integer"
        },
        "totalInPast": {
            "type": "integer"
        },
        "totalInFuture": {
            "type": "integer"
        },
    },
    "required": [
        "before",
        "after",
        "groups",
        "total",
        "totalInPast",
        "totalInFuture",
    ],
    "additionalProperties": False
}

SHERIFF_COMPLAINT_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "ownerName": {
            "type": "string"
        },
        "ownerFullName": {
            "type": ["string", "null"]
        },
        "ownerGender": {
            "type": ["string", "null"]
        },
        "group": to_nullable_object_schema(SHERIFF_COMPLAINT_GROUP_INFO_SCHEMA),
        "reasonCode": {
            "type": "string"
        },
        "reasonDetails": {
            "type": ["string", "null"]
        },
        "anonymousRequested": {
            "type": ["boolean", "null"]
        },
        "createdAt": {
            "type": "integer"
        },
    },
    "required": [
        "id",
        "ownerName",
        "reasonCode",
        "createdAt",
    ],
    "additionalProperties": False
}

SHERIFF_COMPLAINT_INFO_ARRAY_SCHEMA = array_schema(SHERIFF_COMPLAINT_INFO_SCHEMA)

SHERIFF_ORDER_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "delete": {
            "type": ["boolean", "null"]
        },
        "sheriffName": {
            "type": "string"
        },
        "nodeName": {
            "type": "string"
        },
        "nodeFullName": {
            "type": ["string", "null"]
        },
        "feedName": {
            "type": "string"
        },
        "postingId": {
            "type": ["string", "null"]
        },
        "postingRevisionId": {
            "type": ["string", "null"]
        },
        "postingOwnerName": {
            "type": ["string", "null"]
        },
        "postingOwnerFullName": {
            "type": ["string", "null"]
        },
        "postingOwnerGender": {
            "type": ["string", "null"]
        },
        "postingHeading": {
            "type": ["string", "null"]
        },
        "commentId": {
            "type": ["string", "null"]
        },
        "commentRevisionId": {
            "type": ["string", "null"]
        },
        "commentOwnerName": {
            "type": ["string", "null"]
        },
        "commentOwnerFullName": {
            "type": ["string", "null"]
        },
        "commentOwnerGender": {
            "type": ["string", "null"]
        },
        "commentHeading": {
            "type": ["string", "null"]
        },
        "category": {
            "type": "string"
        },
        "reasonCode": {
            "type": ["string", "null"]
        },
        "reasonDetails": {
            "type": ["string", "null"]
        },
        "createdAt": {
            "type": "integer"
        },
        "moment": {
            "type": "integer"
        },
        "signature": {
            "type": "string"
        },
        "signatureVersion": {
            "type": "integer"
        },
        "complaintGroupId": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "id",
        "sheriffName",
        "nodeName",
        "feedName",
        "category",
        "createdAt",
        "moment",
        "signature",
        "signatureVersion",
    ],
    "additionalProperties": False
}

SHERIFF_ORDERS_SLICE_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "before": {
            "type": "integer"
        },
        "after": {
            "type": "integer"
        },
        "orders": {
            "type": "array",
            "items": SHERIFF_ORDER_INFO_SCHEMA
        },
        "total": {
            "type": "integer"
        },
        "totalInPast": {
            "type": "integer"
        },
        "totalInFuture": {
            "type": "integer"
        },
    },
    "required": [
        "before",
        "after",
        "orders",
        "total",
        "totalInPast",
        "totalInFuture",
    ],
    "additionalProperties": False
}

STORY_SUMMARY_BLOCKED_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "operations": {
            "type": "array",
            "items": {
                "type": "string"
            }
        },
        "period": {
            "type": ["integer", "null"]
        },
    },
    "required": [
        "operations",
    ],
    "additionalProperties": False
}

STORY_SUMMARY_FRIEND_GROUP_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": ["string", "null"]
        },
        "title": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

STORY_SUMMARY_ENTRY_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "ownerName": {
            "type": ["string", "null"]
        },
        "ownerFullName": {
            "type": ["string", "null"]
        },
        "ownerGender": {
            "type": ["string", "null"]
        },
        "heading": {
            "type": ["string", "null"]
        },
        "sheriffs": {
            "type": ["array", "null"],
            "items": {
                "type": "string"
            }
        },
        "sheriffMarks": {
            "type": ["array", "null"],
            "items": SHERIFF_MARK_SCHEMA
        },
    },
    "additionalProperties": False
}

STORY_SUMMARY_NODE_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "ownerName": {
            "type": ["string", "null"]
        },
        "ownerFullName": {
            "type": ["string", "null"]
        },
        "ownerGender": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

STORY_SUMMARY_PAGE_CLICKS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "heading": {
            "type": ["string", "null"]
        },
        "href": {
            "type": "string"
        },
        "clicks": {
            "type": "integer"
        },
    },
    "required": [
        "href",
        "clicks",
    ],
    "additionalProperties": False
}

STORY_SUMMARY_REACTION_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "ownerName": {
            "type": ["string", "null"]
        },
        "ownerFullName": {
            "type": ["string", "null"]
        },
        "ownerGender": {
            "type": ["string", "null"]
        },
        "emoji": {
            "type": ["integer", "null"]
        },
    },
    "additionalProperties": False
}

STORY_SUMMARY_SHERIFF_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "sheriffName": {
            "type": "string"
        },
        "orderId": {
            "type": ["string", "null"]
        },
        "complaintId": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "sheriffName",
    ],
    "additionalProperties": False
}

SUBSCRIBER_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "type": {
            "type": "string"
        },
        "feedName": {
            "type": ["string", "null"]
        },
        "postingId": {
            "type": ["string", "null"]
        },
        "nodeName": {
            "type": "string"
        },
        "contact": to_nullable_object_schema(CONTACT_INFO_SCHEMA),
        "createdAt": {
            "type": "integer"
        },
        "operations": to_nullable_object_schema(SUBSCRIBER_OPERATIONS_SCHEMA),
        "ownerOperations": to_nullable_object_schema(SUBSCRIBER_OPERATIONS_SCHEMA),
        "adminOperations": to_nullable_object_schema(SUBSCRIBER_OPERATIONS_SCHEMA),
    },
    "required": [
        "id",
        "type",
        "nodeName",
        "createdAt",
    ],
    "additionalProperties": False
}

SUBSCRIBER_INFO_ARRAY_SCHEMA = array_schema(SUBSCRIBER_INFO_SCHEMA)

SUBSCRIPTION_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "type": {
            "type": "string"
        },
        "feedName": {
            "type": ["string", "null"]
        },
        "remoteNodeName": {
            "type": "string"
        },
        "contact": to_nullable_object_schema(CONTACT_INFO_SCHEMA),
        "remoteFeedName": {
            "type": ["string", "null"]
        },
        "remotePostingId": {
            "type": ["string", "null"]
        },
        "createdAt": {
            "type": "integer"
        },
        "reason": {
            "type": "string"
        },
        "operations": to_nullable_object_schema(SUBSCRIPTION_OPERATIONS_SCHEMA),
    },
    "required": [
        "id",
        "type",
        "remoteNodeName",
        "createdAt",
        "reason",
    ],
    "additionalProperties": False
}

SUBSCRIPTION_INFO_ARRAY_SCHEMA = array_schema(SUBSCRIPTION_INFO_SCHEMA)

TOKEN_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "token": {
            "type": "string"
        },
        "name": {
            "type": ["string", "null"]
        },
        "permissions": {
            "type": ["array", "null"],
            "items": {
                "type": "string"
            }
        },
        "pluginName": {
            "type": ["string", "null"]
        },
        "createdAt": {
            "type": "integer"
        },
        "deadline": {
            "type": ["integer", "null"]
        },
        "lastUsedAt": {
            "type": ["integer", "null"]
        },
        "lastUsedBrowser": {
            "type": ["string", "null"]
        },
        "lastUsedIp": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "id",
        "token",
        "createdAt",
    ],
    "additionalProperties": False
}

TOKEN_INFO_ARRAY_SCHEMA = array_schema(TOKEN_INFO_SCHEMA)

UPDATE_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "important": {
            "type": ["boolean", "null"]
        },
        "description": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

USER_LIST_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "name": {
            "type": "string"
        },
        "total": {
            "type": "integer"
        },
    },
    "required": [
        "name",
        "total",
    ],
    "additionalProperties": False
}

USER_LIST_ITEM_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "nodeName": {
            "type": "string"
        },
        "createdAt": {
            "type": "integer"
        },
        "moment": {
            "type": "integer"
        },
    },
    "required": [
        "nodeName",
        "createdAt",
        "moment",
    ],
    "additionalProperties": False
}

USER_LIST_SLICE_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "listName": {
            "type": "string"
        },
        "before": {
            "type": "integer"
        },
        "after": {
            "type": "integer"
        },
        "items": {
            "type": "array",
            "items": USER_LIST_ITEM_INFO_SCHEMA
        },
        "total": {
            "type": "integer"
        },
        "totalInPast": {
            "type": "integer"
        },
        "totalInFuture": {
            "type": "integer"
        },
    },
    "required": [
        "listName",
        "before",
        "after",
        "items",
        "total",
        "totalInPast",
        "totalInFuture",
    ],
    "additionalProperties": False
}

VERIFICATION_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "correct": {
            "type": "boolean"
        },
        "errorCode": {
            "type": ["string", "null"]
        },
        "errorMessage": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "correct",
    ],
    "additionalProperties": False
}

WHO_AM_I_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "nodeName": {
            "type": ["string", "null"]
        },
        "nodeNameChanging": {
            "type": ["boolean", "null"]
        },
        "fullName": {
            "type": ["string", "null"]
        },
        "gender": {
            "type": ["string", "null"]
        },
        "title": {
            "type": ["string", "null"]
        },
        "avatar": to_nullable_object_schema(AVATAR_IMAGE_SCHEMA),
        "frozen": {
            "type": ["boolean", "null"]
        },
        "type": {
            "type": ["string", "null"]
        },
    },
    "additionalProperties": False
}

ACTIVITY_REACTION_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "remoteNodeName": {
            "type": "string"
        },
        "remoteFullName": {
            "type": ["string", "null"]
        },
        "remoteAvatar": to_nullable_object_schema(AVATAR_IMAGE_SCHEMA),
        "remotePostingId": {
            "type": "string"
        },
        "negative": {
            "type": "boolean"
        },
        "emoji": {
            "type": "integer"
        },
        "createdAt": {
            "type": "integer"
        },
    },
    "required": [
        "remoteNodeName",
        "remotePostingId",
        "negative",
        "emoji",
        "createdAt",
    ],
    "additionalProperties": False
}

ACTIVITY_REACTION_INFO_ARRAY_SCHEMA = array_schema(ACTIVITY_REACTION_INFO_SCHEMA)

BLOCKED_BY_USER_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "blockedOperation": {
            "type": "string"
        },
        "contact": to_nullable_object_schema(CONTACT_INFO_SCHEMA),
        "nodeName": {
            "type": "string"
        },
        "postingId": {
            "type": ["string", "null"]
        },
        "createdAt": {
            "type": "integer"
        },
        "deadline": {
            "type": ["integer", "null"]
        },
        "reason": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "id",
        "blockedOperation",
        "nodeName",
        "createdAt",
    ],
    "additionalProperties": False
}

BLOCKED_BY_USER_INFO_ARRAY_SCHEMA = array_schema(BLOCKED_BY_USER_INFO_SCHEMA)

BLOCKED_USER_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "blockedOperation": {
            "type": "string"
        },
        "nodeName": {
            "type": "string"
        },
        "contact": to_nullable_object_schema(CONTACT_INFO_SCHEMA),
        "entryId": {
            "type": ["string", "null"]
        },
        "entryNodeName": {
            "type": ["string", "null"]
        },
        "entryPostingId": {
            "type": ["string", "null"]
        },
        "createdAt": {
            "type": "integer"
        },
        "deadline": {
            "type": ["integer", "null"]
        },
        "reasonSrc": {
            "type": ["string", "null"]
        },
        "reasonSrcFormat": {
            "type": ["string", "null"]
        },
        "reason": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "id",
        "blockedOperation",
        "nodeName",
        "createdAt",
    ],
    "additionalProperties": False
}

BLOCKED_USER_INFO_ARRAY_SCHEMA = array_schema(BLOCKED_USER_INFO_SCHEMA)

BODY_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "subject": {
            "type": ["string", "null"]
        },
        "text": {
            "type": ["string", "null"]
        },
        "linkPreviews": {
            "type": ["array", "null"],
            "items": LINK_PREVIEW_SCHEMA
        },
    },
    "additionalProperties": False
}

COMMENT_REVISION_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "postingRevisionId": {
            "type": "string"
        },
        "bodyPreview": {
            "type": ["string", "null"]
        },
        "bodySrcHash": {
            "type": "string"
        },
        "bodySrcFormat": {
            "type": ["string", "null"]
        },
        "body": {
            "type": "string"
        },
        "bodyFormat": {
            "type": ["string", "null"]
        },
        "heading": {
            "type": "string"
        },
        "description": {
            "type": ["string", "null"]
        },
        "createdAt": {
            "type": "integer"
        },
        "deletedAt": {
            "type": ["integer", "null"]
        },
        "deadline": {
            "type": ["integer", "null"]
        },
        "digest": {
            "type": ["string", "null"]
        },
        "signature": {
            "type": ["string", "null"]
        },
        "signatureVersion": {
            "type": ["integer", "null"]
        },
        "clientReaction": to_nullable_object_schema(CLIENT_REACTION_INFO_SCHEMA),
        "reactions": to_nullable_object_schema(REACTION_TOTALS_INFO_SCHEMA),
    },
    "required": [
        "id",
        "postingRevisionId",
        "bodySrcHash",
        "body",
        "heading",
        "createdAt",
    ],
    "additionalProperties": False
}

COMMENT_REVISION_INFO_ARRAY_SCHEMA = array_schema(COMMENT_REVISION_INFO_SCHEMA)

CONTACT_WITH_RELATIONSHIPS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "contact": CONTACT_INFO_SCHEMA,
        "subscriber": to_nullable_object_schema(SUBSCRIBER_INFO_SCHEMA),
        "subscription": to_nullable_object_schema(SUBSCRIPTION_INFO_SCHEMA),
        "friend": to_nullable_object_schema(FRIEND_INFO_SCHEMA),
        "friendOf": to_nullable_object_schema(FRIEND_OF_INFO_SCHEMA),
        "blocked": {
            "type": ["array", "null"],
            "items": BLOCKED_USER_INFO_SCHEMA
        },
        "blockedBy": {
            "type": ["array", "null"],
            "items": BLOCKED_BY_USER_INFO_SCHEMA
        },
    },
    "required": [
        "contact",
    ],
    "additionalProperties": False
}

CONTACT_WITH_RELATIONSHIPS_ARRAY_SCHEMA = array_schema(CONTACT_WITH_RELATIONSHIPS_SCHEMA)

FEATURES_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "posting": POSTING_FEATURES_SCHEMA,
        "plugins": {
            "type": ["array", "null"],
            "items": {
                "type": "string"
            }
        },
        "feedWidth": {
            "type": "integer"
        },
        "friendGroups": to_nullable_object_schema(FRIEND_GROUPS_FEATURES_SCHEMA),
        "ask": {
            "type": ["array", "null"],
            "items": {
                "type": "string"
            }
        },
        "subscribed": {
            "type": ["boolean", "null"]
        },
    },
    "required": [
        "posting",
        "feedWidth",
    ],
    "additionalProperties": False
}

FEED_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "feedName": {
            "type": "string"
        },
        "title": {
            "type": ["string", "null"]
        },
        "total": {
            "type": "integer"
        },
        "firstCreatedAt": {
            "type": ["integer", "null"]
        },
        "lastCreatedAt": {
            "type": ["integer", "null"]
        },
        "operations": to_nullable_object_schema(FEED_OPERATIONS_SCHEMA),
        "sheriffs": {
            "type": ["array", "null"],
            "items": {
                "type": "string"
            }
        },
        "sheriffMarks": {
            "type": ["array", "null"],
            "items": SHERIFF_MARK_SCHEMA
        },
    },
    "required": [
        "feedName",
        "total",
    ],
    "additionalProperties": False
}

FEED_INFO_ARRAY_SCHEMA = array_schema(FEED_INFO_SCHEMA)

MEDIA_ATTACHMENT_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "media": to_nullable_object_schema(PRIVATE_MEDIA_FILE_INFO_SCHEMA),
        "remoteMedia": to_nullable_object_schema(REMOTE_MEDIA_INFO_SCHEMA),
        "embedded": {
            "type": "boolean"
        },
    },
    "required": [
        "embedded",
    ],
    "additionalProperties": False
}

POSTING_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "revisionId": {
            "type": "string"
        },
        "receiverRevisionId": {
            "type": ["string", "null"]
        },
        "totalRevisions": {
            "type": "integer"
        },
        "receiverName": {
            "type": ["string", "null"]
        },
        "receiverFullName": {
            "type": ["string", "null"]
        },
        "receiverGender": {
            "type": ["string", "null"]
        },
        "receiverAvatar": to_nullable_object_schema(AVATAR_IMAGE_SCHEMA),
        "receiverPostingId": {
            "type": ["string", "null"]
        },
        "parentMediaId": {
            "type": ["string", "null"]
        },
        "ownerName": {
            "type": "string"
        },
        "ownerFullName": {
            "type": ["string", "null"]
        },
        "ownerGender": {
            "type": ["string", "null"]
        },
        "ownerAvatar": to_nullable_object_schema(AVATAR_IMAGE_SCHEMA),
        "bodyPreview": {
            "type": ["string", "null"]
        },
        "bodySrc": {
            "type": ["string", "null"]
        },
        "bodySrcHash": {
            "type": "string"
        },
        "bodySrcFormat": {
            "type": ["string", "null"]
        },
        "body": {
            "type": "string"
        },
        "bodyFormat": {
            "type": ["string", "null"]
        },
        "media": {
            "type": ["array", "null"],
            "items": MEDIA_ATTACHMENT_SCHEMA
        },
        "heading": {
            "type": "string"
        },
        "description": {
            "type": ["string", "null"]
        },
        "updateInfo": to_nullable_object_schema(UPDATE_INFO_SCHEMA),
        "createdAt": {
            "type": "integer"
        },
        "editedAt": {
            "type": ["integer", "null"]
        },
        "deletedAt": {
            "type": ["integer", "null"]
        },
        "receiverCreatedAt": {
            "type": ["integer", "null"]
        },
        "receiverEditedAt": {
            "type": ["integer", "null"]
        },
        "receiverDeletedAt": {
            "type": ["integer", "null"]
        },
        "revisionCreatedAt": {
            "type": "integer"
        },
        "receiverRevisionCreatedAt": {
            "type": ["integer", "null"]
        },
        "deadline": {
            "type": ["integer", "null"]
        },
        "digest": {
            "type": ["string", "null"]
        },
        "signature": {
            "type": ["string", "null"]
        },
        "signatureVersion": {
            "type": ["integer", "null"]
        },
        "feedReferences": {
            "type": ["array", "null"],
            "items": FEED_REFERENCE_SCHEMA
        },
        "blockedInstants": {
            "type": ["array", "null"],
            "items": BLOCKED_POSTING_INSTANT_INFO_SCHEMA
        },
        "operations": to_nullable_object_schema(POSTING_OPERATIONS_SCHEMA),
        "receiverOperations": to_nullable_object_schema(POSTING_OPERATIONS_SCHEMA),
        "commentOperations": to_nullable_object_schema(COMMENT_OPERATIONS_SCHEMA),
        "reactionOperations": to_nullable_object_schema(REACTION_OPERATIONS_SCHEMA),
        "commentReactionOperations": to_nullable_object_schema(REACTION_OPERATIONS_SCHEMA),
        "blockedOperations": {
            "type": ["array", "null"],
            "items": {
                "type": "string"
            }
        },
        "blockedCommentOperations": {
            "type": ["array", "null"],
            "items": {
                "type": "string"
            }
        },
        "sheriffs": {
            "type": ["array", "null"],
            "items": {
                "type": "string"
            }
        },
        "sheriffMarks": {
            "type": ["array", "null"],
            "items": SHERIFF_MARK_SCHEMA
        },
        "rejectedReactions": to_nullable_object_schema(REJECTED_REACTIONS_SCHEMA),
        "commentRejectedReactions": to_nullable_object_schema(REJECTED_REACTIONS_SCHEMA),
        "clientReaction": to_nullable_object_schema(CLIENT_REACTION_INFO_SCHEMA),
        "reactions": to_nullable_object_schema(REACTION_TOTALS_INFO_SCHEMA),
        "sources": {
            "type": ["array", "null"],
            "items": POSTING_SOURCE_INFO_SCHEMA
        },
        "totalComments": {
            "type": ["integer", "null"]
        },
        "recommended": {
            "type": ["boolean", "null"]
        },
        "externalSourceUri": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "id",
        "revisionId",
        "totalRevisions",
        "ownerName",
        "bodySrcHash",
        "body",
        "heading",
        "createdAt",
        "revisionCreatedAt",
    ],
    "additionalProperties": False
}

POSTING_INFO_ARRAY_SCHEMA = array_schema(POSTING_INFO_SCHEMA)

POSTING_REVISION_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "receiverId": {
            "type": ["string", "null"]
        },
        "bodyPreview": {
            "type": ["string", "null"]
        },
        "bodySrcHash": {
            "type": "string"
        },
        "bodySrcFormat": {
            "type": ["string", "null"]
        },
        "body": {
            "type": "string"
        },
        "bodyFormat": {
            "type": ["string", "null"]
        },
        "media": {
            "type": ["array", "null"],
            "items": MEDIA_ATTACHMENT_SCHEMA
        },
        "heading": {
            "type": "string"
        },
        "description": {
            "type": ["string", "null"]
        },
        "updateInfo": to_nullable_object_schema(UPDATE_INFO_SCHEMA),
        "createdAt": {
            "type": "integer"
        },
        "deletedAt": {
            "type": ["integer", "null"]
        },
        "receiverCreatedAt": {
            "type": ["integer", "null"]
        },
        "receiverDeletedAt": {
            "type": ["integer", "null"]
        },
        "digest": {
            "type": ["string", "null"]
        },
        "signature": {
            "type": ["string", "null"]
        },
        "signatureVersion": {
            "type": ["integer", "null"]
        },
        "clientReaction": to_nullable_object_schema(CLIENT_REACTION_INFO_SCHEMA),
        "reactions": to_nullable_object_schema(REACTION_TOTALS_INFO_SCHEMA),
    },
    "required": [
        "id",
        "bodySrcHash",
        "body",
        "heading",
        "createdAt",
    ],
    "additionalProperties": False
}

POSTING_REVISION_INFO_ARRAY_SCHEMA = array_schema(POSTING_REVISION_INFO_SCHEMA)

REACTION_CREATED_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "reaction": to_nullable_object_schema(REACTION_INFO_SCHEMA),
        "totals": REACTION_TOTALS_INFO_SCHEMA,
    },
    "required": [
        "totals",
    ],
    "additionalProperties": False
}

SEARCH_ENTRY_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "nodeName": {
            "type": "string"
        },
        "postingId": {
            "type": "string"
        },
        "commentId": {
            "type": ["string", "null"]
        },
        "ownerName": {
            "type": "string"
        },
        "ownerFullName": {
            "type": ["string", "null"]
        },
        "ownerAvatar": to_nullable_object_schema(AVATAR_IMAGE_SCHEMA),
        "bodyPreview": {
            "type": "string"
        },
        "bodyFormat": {
            "type": ["string", "null"]
        },
        "heading": {
            "type": "string"
        },
        "imageCount": {
            "type": ["integer", "null"]
        },
        "videoPresent": {
            "type": ["boolean", "null"]
        },
        "mediaPreview": to_nullable_object_schema(PUBLIC_MEDIA_FILE_INFO_SCHEMA),
        "mediaPreviewId": {
            "type": ["string", "null"]
        },
        "repliedTo": to_nullable_object_schema(SEARCH_REPLIED_TO_SCHEMA),
        "createdAt": {
            "type": "integer"
        },
        "operations": to_nullable_object_schema(SEARCH_ENTRY_OPERATIONS_SCHEMA),
        "moment": {
            "type": "integer"
        },
    },
    "required": [
        "nodeName",
        "postingId",
        "ownerName",
        "bodyPreview",
        "heading",
        "createdAt",
        "moment",
    ],
    "additionalProperties": False
}

SEARCH_HASHTAG_SLICE_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "before": {
            "type": "integer"
        },
        "after": {
            "type": "integer"
        },
        "entries": {
            "type": "array",
            "items": SEARCH_ENTRY_INFO_SCHEMA
        },
    },
    "required": [
        "before",
        "after",
        "entries",
    ],
    "additionalProperties": False
}

SEARCH_TEXT_PAGE_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "page": {
            "type": "integer"
        },
        "total": {
            "type": "integer"
        },
        "entries": {
            "type": "array",
            "items": SEARCH_ENTRY_INFO_SCHEMA
        },
    },
    "required": [
        "page",
        "total",
        "entries",
    ],
    "additionalProperties": False
}

SETTING_TYPE_MODIFIERS_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "format": {
            "type": ["string", "null"]
        },
        "min": {
            "type": ["string", "null"]
        },
        "max": {
            "type": ["string", "null"]
        },
        "multiline": {
            "type": ["boolean", "null"]
        },
        "never": {
            "type": ["boolean", "null"]
        },
        "always": {
            "type": ["boolean", "null"]
        },
        "items": {
            "type": ["array", "null"],
            "items": SETTING_VALUE_CHOICE_SCHEMA
        },
        "principals": {
            "type": ["array", "null"],
            "items": {
                "type": "string"
            }
        },
    },
    "additionalProperties": False
}

STORY_SUMMARY_DATA_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "node": to_nullable_object_schema(STORY_SUMMARY_NODE_SCHEMA),
        "posting": to_nullable_object_schema(STORY_SUMMARY_ENTRY_SCHEMA),
        "comment": to_nullable_object_schema(STORY_SUMMARY_ENTRY_SCHEMA),
        "comments": {
            "type": ["array", "null"],
            "items": STORY_SUMMARY_ENTRY_SCHEMA
        },
        "totalComments": {
            "type": ["integer", "null"]
        },
        "repliedTo": to_nullable_object_schema(STORY_SUMMARY_ENTRY_SCHEMA),
        "parentPosting": to_nullable_object_schema(STORY_SUMMARY_ENTRY_SCHEMA),
        "reaction": to_nullable_object_schema(STORY_SUMMARY_REACTION_SCHEMA),
        "reactions": {
            "type": ["array", "null"],
            "items": STORY_SUMMARY_REACTION_SCHEMA
        },
        "totalReactions": {
            "type": ["integer", "null"]
        },
        "feedName": {
            "type": ["string", "null"]
        },
        "subscriptionReason": {
            "type": ["string", "null"]
        },
        "friendGroup": to_nullable_object_schema(STORY_SUMMARY_FRIEND_GROUP_SCHEMA),
        "blocked": to_nullable_object_schema(STORY_SUMMARY_BLOCKED_SCHEMA),
        "sheriff": to_nullable_object_schema(STORY_SUMMARY_SHERIFF_SCHEMA),
        "description": {
            "type": ["string", "null"]
        },
        "clicks": {
            "type": ["array", "null"],
            "items": STORY_SUMMARY_PAGE_CLICKS_SCHEMA
        },
    },
    "additionalProperties": False
}

COMMENT_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "ownerName": {
            "type": "string"
        },
        "ownerFullName": {
            "type": ["string", "null"]
        },
        "ownerGender": {
            "type": ["string", "null"]
        },
        "ownerAvatar": to_nullable_object_schema(AVATAR_IMAGE_SCHEMA),
        "postingId": {
            "type": "string"
        },
        "postingRevisionId": {
            "type": "string"
        },
        "revisionId": {
            "type": "string"
        },
        "totalRevisions": {
            "type": "integer"
        },
        "bodyPreview": {
            "type": ["string", "null"]
        },
        "bodySrc": {
            "type": ["string", "null"]
        },
        "bodySrcHash": {
            "type": "string"
        },
        "bodySrcFormat": {
            "type": ["string", "null"]
        },
        "body": {
            "type": "string"
        },
        "bodyFormat": {
            "type": ["string", "null"]
        },
        "media": {
            "type": ["array", "null"],
            "items": MEDIA_ATTACHMENT_SCHEMA
        },
        "heading": {
            "type": "string"
        },
        "description": {
            "type": ["string", "null"]
        },
        "repliedTo": to_nullable_object_schema(REPLIED_TO_SCHEMA),
        "moment": {
            "type": "integer"
        },
        "createdAt": {
            "type": "integer"
        },
        "editedAt": {
            "type": ["integer", "null"]
        },
        "deletedAt": {
            "type": ["integer", "null"]
        },
        "revisionCreatedAt": {
            "type": "integer"
        },
        "deadline": {
            "type": ["integer", "null"]
        },
        "digest": {
            "type": ["string", "null"]
        },
        "signature": {
            "type": ["string", "null"]
        },
        "signatureVersion": {
            "type": ["integer", "null"]
        },
        "operations": to_nullable_object_schema(COMMENT_OPERATIONS_SCHEMA),
        "reactionOperations": to_nullable_object_schema(REACTION_OPERATIONS_SCHEMA),
        "ownerOperations": to_nullable_object_schema(COMMENT_OPERATIONS_SCHEMA),
        "seniorOperations": to_nullable_object_schema(COMMENT_OPERATIONS_SCHEMA),
        "blockedOperations": {
            "type": ["array", "null"],
            "items": {
                "type": "string"
            }
        },
        "sheriffMarks": {
            "type": ["array", "null"],
            "items": SHERIFF_MARK_SCHEMA
        },
        "rejectedReactions": to_nullable_object_schema(REJECTED_REACTIONS_SCHEMA),
        "ownerRejectedReactions": to_nullable_object_schema(REJECTED_REACTIONS_SCHEMA),
        "seniorRejectedReactions": to_nullable_object_schema(REJECTED_REACTIONS_SCHEMA),
        "clientReaction": to_nullable_object_schema(CLIENT_REACTION_INFO_SCHEMA),
        "seniorReaction": to_nullable_object_schema(CLIENT_REACTION_INFO_SCHEMA),
        "reactions": to_nullable_object_schema(REACTION_TOTALS_INFO_SCHEMA),
    },
    "required": [
        "id",
        "ownerName",
        "postingId",
        "postingRevisionId",
        "revisionId",
        "totalRevisions",
        "bodySrcHash",
        "body",
        "heading",
        "moment",
        "createdAt",
        "revisionCreatedAt",
    ],
    "additionalProperties": False
}

COMMENTS_SLICE_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "before": {
            "type": "integer"
        },
        "after": {
            "type": "integer"
        },
        "comments": {
            "type": "array",
            "items": COMMENT_INFO_SCHEMA
        },
        "total": {
            "type": "integer"
        },
        "totalInPast": {
            "type": "integer"
        },
        "totalInFuture": {
            "type": "integer"
        },
    },
    "required": [
        "before",
        "after",
        "comments",
        "total",
        "totalInPast",
        "totalInFuture",
    ],
    "additionalProperties": False
}

DRAFT_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "draftType": {
            "type": "string"
        },
        "receiverName": {
            "type": "string"
        },
        "receiverPostingId": {
            "type": ["string", "null"]
        },
        "receiverCommentId": {
            "type": ["string", "null"]
        },
        "repliedToId": {
            "type": ["string", "null"]
        },
        "createdAt": {
            "type": "integer"
        },
        "editedAt": {
            "type": ["integer", "null"]
        },
        "deadline": {
            "type": ["integer", "null"]
        },
        "ownerFullName": {
            "type": ["string", "null"]
        },
        "ownerAvatar": to_nullable_object_schema(AVATAR_IMAGE_SCHEMA),
        "rejectedReactions": to_nullable_object_schema(REJECTED_REACTIONS_SCHEMA),
        "commentRejectedReactions": to_nullable_object_schema(REJECTED_REACTIONS_SCHEMA),
        "bodySrc": {
            "type": ["string", "null"]
        },
        "bodySrcFormat": {
            "type": ["string", "null"]
        },
        "body": {
            "type": "string"
        },
        "bodyFormat": {
            "type": ["string", "null"]
        },
        "media": {
            "type": ["array", "null"],
            "items": MEDIA_ATTACHMENT_SCHEMA
        },
        "heading": {
            "type": "string"
        },
        "publishAt": {
            "type": ["integer", "null"]
        },
        "updateInfo": to_nullable_object_schema(UPDATE_INFO_SCHEMA),
        "operations": to_nullable_object_schema(POSTING_OPERATIONS_SCHEMA),
        "commentOperations": to_nullable_object_schema(COMMENT_OPERATIONS_SCHEMA),
    },
    "required": [
        "id",
        "draftType",
        "receiverName",
        "createdAt",
        "body",
        "heading",
    ],
    "additionalProperties": False
}

DRAFT_INFO_ARRAY_SCHEMA = array_schema(DRAFT_INFO_SCHEMA)

ENTRY_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "posting": to_nullable_object_schema(POSTING_INFO_SCHEMA),
        "comment": to_nullable_object_schema(COMMENT_INFO_SCHEMA),
    },
    "additionalProperties": False
}

ENTRY_INFO_ARRAY_SCHEMA = array_schema(ENTRY_INFO_SCHEMA)

SETTING_META_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "name": {
            "type": "string"
        },
        "type": {
            "type": "string"
        },
        "defaultValue": {
            "type": ["string", "null"]
        },
        "privileged": {
            "type": ["boolean", "null"]
        },
        "title": {
            "type": "string"
        },
        "modifiers": to_nullable_object_schema(SETTING_TYPE_MODIFIERS_SCHEMA),
    },
    "required": [
        "name",
        "type",
        "title",
    ],
    "additionalProperties": False
}

SETTING_META_INFO_ARRAY_SCHEMA = array_schema(SETTING_META_INFO_SCHEMA)

STORY_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "id": {
            "type": "string"
        },
        "feedName": {
            "type": "string"
        },
        "storyType": {
            "type": "string"
        },
        "createdAt": {
            "type": "integer"
        },
        "publishedAt": {
            "type": "integer"
        },
        "pinned": {
            "type": ["boolean", "null"]
        },
        "moment": {
            "type": "integer"
        },
        "viewed": {
            "type": ["boolean", "null"]
        },
        "read": {
            "type": ["boolean", "null"]
        },
        "satisfied": {
            "type": ["boolean", "null"]
        },
        "summaryNodeName": {
            "type": ["string", "null"]
        },
        "summaryFullName": {
            "type": ["string", "null"]
        },
        "summaryAvatar": to_nullable_object_schema(AVATAR_IMAGE_SCHEMA),
        "summary": {
            "type": ["string", "null"]
        },
        "summaryData": to_nullable_object_schema(STORY_SUMMARY_DATA_SCHEMA),
        "posting": to_nullable_object_schema(POSTING_INFO_SCHEMA),
        "postingId": {
            "type": ["string", "null"]
        },
        "comment": to_nullable_object_schema(COMMENT_INFO_SCHEMA),
        "commentId": {
            "type": ["string", "null"]
        },
        "remoteNodeName": {
            "type": ["string", "null"]
        },
        "remoteFullName": {
            "type": ["string", "null"]
        },
        "remotePostingId": {
            "type": ["string", "null"]
        },
        "remoteCommentId": {
            "type": ["string", "null"]
        },
        "remoteMediaId": {
            "type": ["string", "null"]
        },
        "operations": to_nullable_object_schema(STORY_OPERATIONS_SCHEMA),
    },
    "required": [
        "id",
        "feedName",
        "storyType",
        "createdAt",
        "publishedAt",
        "moment",
    ],
    "additionalProperties": False
}

COMMENT_CREATED_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "comment": COMMENT_INFO_SCHEMA,
        "total": {
            "type": "integer"
        },
    },
    "required": [
        "comment",
        "total",
    ],
    "additionalProperties": False
}

FEED_SLICE_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "before": {
            "type": "integer"
        },
        "after": {
            "type": "integer"
        },
        "stories": {
            "type": "array",
            "items": STORY_INFO_SCHEMA
        },
        "totalInPast": {
            "type": "integer"
        },
        "totalInFuture": {
            "type": "integer"
        },
    },
    "required": [
        "before",
        "after",
        "stories",
        "totalInPast",
        "totalInFuture",
    ],
    "additionalProperties": False
}

PLUGIN_INFO_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "nodeId": {
            "type": "string"
        },
        "local": {
            "type": "boolean"
        },
        "name": {
            "type": "string"
        },
        "title": {
            "type": ["string", "null"]
        },
        "description": {
            "type": ["string", "null"]
        },
        "location": {
            "type": ["string", "null"]
        },
        "acceptedEvents": {
            "type": ["array", "null"],
            "items": {
                "type": "string"
            }
        },
        "settings": {
            "type": ["array", "null"],
            "items": SETTING_META_INFO_SCHEMA
        },
        "tokenId": {
            "type": ["string", "null"]
        },
    },
    "required": [
        "nodeId",
        "local",
        "name",
    ],
    "additionalProperties": False
}

PLUGIN_INFO_ARRAY_SCHEMA = array_schema(PLUGIN_INFO_SCHEMA)

PUSH_CONTENT_SCHEMA: Any = {
    "type": "object",
    "properties": {
        "type": {
            "type": "string"
        },
        "id": {
            "type": ["string", "null"]
        },
        "story": to_nullable_object_schema(STORY_INFO_SCHEMA),
        "feedStatus": to_nullable_object_schema(FEED_WITH_STATUS_SCHEMA),
    },
    "required": [
        "type",
    ],
    "additionalProperties": False
}

PUSH_CONTENT_ARRAY_SCHEMA = array_schema(PUSH_CONTENT_SCHEMA)



import json
from copy import deepcopy
from enum import Enum
from typing import Any, IO, Mapping, Sequence

import requests
from jsonschema.exceptions import ValidationError

from . import schemas
from .types import Result, BodyFormat, SourceFormat, Body
from ..pruning_validator import validate
from ..structure import Structure


class MoeraNodeError(Exception):
    """Generic node error."""

    def __init__(self, name: str, message: str):
        """
        :param name: request name
        :param message: error message
        """
        super().__init__(name + ': Node error: ' + message)


class MoeraNodeApiError(MoeraNodeError):
    """Node returned an error response."""
    error_code: str
    """Error code."""

    def __init__(self, name: str, result: Result):
        """
        :param name: request name
        :param result: node response
        """
        super().__init__(name, result.message if result.message is not None else '')
        self.error_code = result.error_code


class MoeraNodeConnectionError(Exception):
    """Error while connecting the node."""

    def __init__(self, message: str):
        """
        :param message: error message
        """
        super().__init__('Node connection error: ' + message)


def encode_body(decoded: Body, format: BodyFormat | SourceFormat | None) -> str:
    if format is not None and format.lower() == "application":
        return decoded.text
    return json.dumps(decoded.json(), indent=None)


def encode_bodies(data: Structure | Sequence[Structure]) -> Structure | list[Structure]:
    if isinstance(data, Sequence):
        return [encode_bodies(item) for item in data]
    encoded = deepcopy(data)
    if getattr(data, 'body', None) is not None:
        encoded.body = encode_body(getattr(data, 'body'), getattr(data, 'body_format', None))
    if getattr(data, 'body_preview', None) is not None:
        encoded.body_preview = encode_body(getattr(data, 'body_preview'), getattr(data, 'body_format', None))
    if getattr(data, 'body_src', None) is not None:
        encoded.body_src = encode_body(getattr(data, 'body_src'), getattr(data, 'body_src_format', None))
    return encoded


def decode_body(name: str, encoded: str, format: BodyFormat | SourceFormat | None) -> Body:
    if format is not None and format.lower() == "application":
        return Body(text=encoded)
    try:
        body = json.loads(encoded)
        validate(body, schema=schemas.BODY_SCHEMA)
        return Body.from_json(body)
    except json.JSONDecodeError as e:
        raise MoeraNodeError(name, 'Invalid body encoding') from e
    except ValidationError as e:
        raise MoeraNodeError(name, 'Invalid body: ' + repr(e)) from e


def decode_bodies(name: str, data):
    if isinstance(data, Sequence):
        return [decode_bodies(name, item) for item in data]
    decoded = deepcopy(data)
    if data.get('stories', None) is not None:
        decoded['stories'] = [decode_bodies(name, item) for item in data['stories']]
    if data.get('comments', None) is not None:
        decoded['comments'] = [decode_bodies(name, item) for item in data['comments']]
    if data.get('comment', None) is not None:
        decoded['comment'] = decode_bodies(name, data['comment'])
    if data.get('posting', None) is not None:
        decoded['posting'] = decode_bodies(name, data['posting'])
    if data.get('body', None) is not None:
        decoded['body'] = decode_body(name, data['body'], data.get('bodyFormat', None))
    if data.get('bodyPreview', None) is not None:
        decoded['bodyPreview'] = decode_body(name, data['bodyPreview'], data.get('bodyFormat', None))
    if data.get('bodySrc', None) is not None:
        decoded['bodySrc'] = decode_body(name, data['bodySrc'], data.get('bodySrcFormat', None))
    return decoded


class NodeAuth(Enum):
    """Authentication type."""
    NONE = 0
    """No authentication."""
    PEER = 1
    """Carte authentication."""
    ADMIN = 2
    """Admin token authentication."""
    ROOT_ADMIN = 3
    """Root admin secret authentication."""


class CarteSource:
    def get_carte(self) -> str:
        raise NotImplemented


FETCH_TIMEOUT = 10 # seconds
UPDATE_TIMEOUT = 60 # seconds
LARGE_BODY_MIN = 65536

def timeout_duration(method: str, body_encoded: str | None, body_file: IO | None) -> int | None:
    if method == "POST" or method == "PUT":
        large_body = body_file is not None or body_encoded is not None and len(body_encoded) > LARGE_BODY_MIN
        duration =  UPDATE_TIMEOUT if large_body else FETCH_TIMEOUT
    else:
        duration = FETCH_TIMEOUT

    return duration

class Caller:
    root: str | None = None
    """API endpoint URL of the node."""
    _root_secret: str | None = None
    _token: str | None = None
    _carte: str | None = None
    _carte_source: CarteSource | None = None
    _auth_method: NodeAuth = NodeAuth.NONE

    def node_url(self, url: str) -> None:
        """Set node URL."""
        self.root = moera_root(url)

    def root_secret(self, secret: str) -> None:
        """Set root secret for authentication."""
        self._root_secret = secret

    def token(self, token: str) -> None:
        """Set admin token for authentication."""
        self._token = token

    def carte(self, carte: str) -> None:
        """Set carte for authentication."""
        self._carte = carte

    def carte_source(self, carte_source: CarteSource) -> None:
        """Set a source of cartes for authentication."""
        self._carte_source = carte_source

    def auth_method(self, auth_method: NodeAuth) -> None:
        """Select authentication method for the following requests."""
        self._auth_method = auth_method

    def no_auth(self) -> None:
        """Switch off authentication for the following requests."""
        self.auth_method(NodeAuth.NONE)

    def auth(self) -> None:
        """Select carte authentication for the following requests."""
        self.auth_method(NodeAuth.PEER)

    def auth_admin(self) -> None:
        """Select admin token authentication for the following requests."""
        self.auth_method(NodeAuth.ADMIN)

    def auth_root_admin(self) -> None:
        """Select root admin secret authentication for the following requests."""
        self.auth_method(NodeAuth.ROOT_ADMIN)

    def call(self, name: str, location: str, params: Mapping[str, str | int | None] | None = None, method: str = 'GET',
             body: Structure | Sequence[Structure] | None = None, body_file: IO | None = None,
             body_file_type: str | None = None, auth: bool = True, schema: Any = None,
             bodies: bool = False, src_bodies: bool = False):
        """
        Generic method for making node API requests.

        :param name: request name (for error messages)
        :param location: request path
        :param params: query parameters, mapping name to value, ``None`` values are skipped
        :param method: request method (one of 'GET', 'POST', 'PUT', 'DELETE')
        :param body: request body
        :param body_file: file to read the request body from
        :param body_file_type: content-type of the request body, when read from a file
        :param auth: `True` to authenticate the request, `False` otherwise
        :param schema: JSON schema to validate the response
        :param bodies: `True` to decode `Body` structures in the response, `False` otherwise
        :param src_bodies: `True` to encode `Body` structures in the request, `False` otherwise
        :return: the decoded response
        """
        response = None
        try:
            body_encoded = None
            if body is not None:
                if src_bodies:
                    body = encode_bodies(body)
                if isinstance(body, Sequence):
                    body_encoded = [b.json() for b in body]
                else:
                    body_encoded = body.json()

            headers = {
                'Accept': 'application/json',
                'Content-Type': 'application/json' if body_file_type is None else body_file_type
            }
            bearer: str | None = None
            if auth:
                match self._auth_method:
                    case NodeAuth.PEER:
                        if self._carte_source is not None:
                            bearer = 'carte:' + self._carte_source.get_carte()
                        elif self._carte is not None:
                            bearer = 'carte:' + self._carte
                        else:
                            raise ValueError('Carte is not set')
                    case NodeAuth.ADMIN:
                        if self._token is None:
                            raise ValueError('Token is not set')
                        bearer = 'token:' + self._token
                    case NodeAuth.ROOT_ADMIN:
                        if self._root_secret is None:
                            raise ValueError('Root secret is not set')
                        bearer = 'secret:' + self._root_secret
            if bearer is not None:
                headers['Authorization'] = f'Bearer {bearer}'

            if self.root is None:
                raise ValueError('Node URL is not set')

            r = requests.request(
                method=method,
                url=self.root + '/api' + location,
                params=params,
                headers=headers,
                json=body_encoded,
                data=body_file,
                stream=schema == "blob",
                timeout=timeout_duration(method, body_encoded, body_file)
            )

            response = r.json()
            if r.status_code not in [200, 201]:
                validate(response, schema=schemas.RESULT_SCHEMA)
                raise MoeraNodeApiError(name, Result.from_json(response))
            if schema is not None and schema != "blob":
                validate(response, schema=schema)

            if bodies:
                return decode_bodies(name, response)
            else:
                return response
        except requests.exceptions.RequestException as e:
            raise MoeraNodeConnectionError(str(e)) from e
        except (requests.exceptions.InvalidJSONError, ValidationError) as e:
            message = f'Invalid server response: {repr(e)}'
            if response is not None:
                message += f'\nResponse:\n{response}'
            raise MoeraNodeError(name, message) from e


def moera_root(url: str) -> str:
    """
    Convert partial node URL to a standardized form.

    :param url: partial URL
    :return: standard URL
    """
    if not url.startswith('http:') and not url.startswith('https:'):
        url = 'https://' + url
    url = url.removesuffix('/').removesuffix('/api')
    if not url.endswith('/moera'):
        url += '/moera'
    return url



from .crypto import (digest_fingerprint, generate_key, generate_mnemonic_key, mnemonic_to_private_key, raw_private_key,
                     raw_public_key, raw_to_private_key, digest_fingerprint, sign_fingerprint,
                     verify_fingerprint_signature)



from __future__ import annotations

from urllib.parse import quote, unquote, urlparse, urlunparse

from .naming.naming import shorten

REDIRECTOR = 'moera.page'


class UniversalLocation:
    """Represents location part of a universal Moera URL."""

    _node_name: str | None
    _scheme: str
    _authority: str | None
    _path: str | None

    query: str | None
    """Query component of the URL."""

    fragment: str | None
    """Fragment identifier of the URL."""

    def __init__(self, node_name: str | None = None, scheme: str | None = None, authority: str | None = None,
                 path: str | None = None, query: str | None = None, fragment: str | None = None):
        """
        :param node_name: the node name
        :param scheme: scheme specifier of the node location (``'https'``, if set to `None` or empty)
        :param authority: authority (host name and optional port) of the node location
        :param path: virtual path at the node (``'/'``, if set to `None` or empty)
        :param query: query component of the URL (without ``?``)
        :param fragment: fragment identifier of the URL (without ``#``)
        """
        self.node_name = node_name
        self.scheme = scheme
        self.authority = authority
        self.path = path
        self.query = query
        self.fragment = fragment

    @property
    def node_name(self) -> str | None:
        """
        The node name.
        """
        return self._node_name

    @node_name.setter
    def node_name(self, node_name: str | None) -> None:
        self._node_name = shorten(node_name)

    @property
    def scheme(self) -> str:
        """
        Scheme specifier of the node location.
        """
        return self._scheme

    @scheme.setter
    def scheme(self, scheme: str | None) -> None:
        if scheme is None or scheme == '':
            self._scheme = 'https'
        else:
            self._scheme = scheme

    @property
    def authority(self) -> str | None:
        """
        Authority (host name and optional port) of the node location.
        """
        return self._authority

    @authority.setter
    def authority(self, authority: str | None) -> None:
        if authority is None or authority == '':
            self._authority = None
        else:
            self._authority = authority

    @property
    def path(self) -> str | None:
        """
        Virtual path at the node.
        """
        return self._path

    @path.setter
    def path(self, path: str | None) -> None:
        if path is not None and path.startswith('/moera'):
            path = path[6:]
        if path is not None and path != '':
            self._path = path
        else:
            self._path = '/'

    @property
    def location(self) -> str:
        """
        Universal Moera location (without query and fragment).
        """
        loc = '/@'
        if self._node_name is not None:
            loc += quote(self._node_name, safe='')
        loc += '/'
        if self._authority is not None:
            if self._scheme is not None and self._scheme.lower() != 'https':
                loc += self._scheme + ':'
            loc += self._authority
        else:
            loc += '~'
        loc += self._path
        return loc

    def __str__(self) -> str:
        return str(urlunparse(('', '', self.location, '', self.query, self.fragment)))

    def __repr__(self):
        return repr((self._node_name, self._scheme, self._authority, self._path, self.query, self.fragment))


def parse(url: str | None) -> UniversalLocation:
    """
    Parse the location part (including query and fragment) of a universal URL.

    :param url: the URL to be parsed
    :return: the parsed location
    """
    if url is None:
        return UniversalLocation()

    url_parts = urlparse(url)

    path = url_parts.path.removeprefix('/').removesuffix('/')
    if path == '':
        return UniversalLocation()

    dirs = path.split('/')
    if not dirs[0].startswith('@'):
        return UniversalLocation()

    node_name = None
    if len(dirs[0]) > 1:
        node_name = unquote(dirs[0][1:])

    scheme = None
    authority = None
    host = None
    port = None

    if len(dirs) > 1 and dirs[1] != '~':
        parts = dirs[1].split(':')
        i = 0
        if '.' not in parts[i]:
            scheme = parts[i]
            i += 1
        if i < len(parts):
            host = parts[i]
            i += 1
        if i < len(parts):
            port = parts[i]

    if host is not None and host != '':
        authority = host
        if port is not None and port != '':
            authority += ':' + port

    path = ''
    for dir in dirs[2:]:
        path += '/' + dir
    if path == '':
        path = '/'

    return UniversalLocation(node_name, scheme, authority, path, url_parts.query, url_parts.fragment)


def redirect_to_url(node_name: str | None, url: str | None = None) -> str:
    """
    Build a universal Moera URL from the direct URL of a page on a node, adding the node name provided.

    :param node_name: the node name
    :param url: the direct URL
    :return: the universal URL
    """
    try:
        if url is not None:
            url_parts = urlparse(url)
            uni = UniversalLocation(node_name, url_parts.scheme, url_parts.netloc, url_parts.path, url_parts.query,
                                    url_parts.fragment)
        else:
            uni = UniversalLocation(node_name)
        return str(urlunparse(('https', REDIRECTOR, uni.location, '', uni.query, uni.fragment)))
    except ValueError:
        return 'https://' + REDIRECTOR


def redirect_to(node_name: str | None, root_url: str | None, path: str | None = None, query: str | None = None,
                fragment: str | None = None) -> str:
    """
    Build a universal Moera URL from the node name, the Moera root URL of the node, virtual path and other components.

    :param node_name: the node name
    :param root_url: the Moera root URL of the node
    :param path: virtual path at the node (``'/'``, if set to `None` or empty)
    :param query: query component of the URL
    :param fragment: fragment identifier of the URL
    :return: the universal URL
    """
    try:
        if root_url is not None:
            url_parts = urlparse(root_url)
            uni = UniversalLocation(node_name, url_parts.scheme, url_parts.netloc, path, query, fragment)
        else:
            uni = UniversalLocation(node_name, None, None, path, query, fragment)
        return str(urlunparse(('https', REDIRECTOR, uni.location, '', uni.query, uni.fragment)))
    except ValueError:
        return 'https://' + REDIRECTOR



from ..crypto.crypto import fingerprint_bytes
from ..crypto.fingerprint import FingerprintSchema

PUT_CALL_FINGERPRINT0_SCHEMA: FingerprintSchema = [
    ('version', 'number'),
    ('name', 'string'),
    ('generation', 'number'),
    ('updating_key', 'bytes'),
    ('node_uri', 'string'),
    ('signing_key', 'bytes'),
    ('valid_from', 'number'),
    ('previous_digest', 'bytes'),
]


def create_put_call_fingerprint0(name: str, generation: int, updating_key: bytes, node_uri: str, signing_key: bytes,
                                 valid_from: int, previous_digest: bytes | None) -> bytes:
    return fingerprint_bytes({'version': 0} | locals(), PUT_CALL_FINGERPRINT0_SCHEMA)



from base64 import b64encode
from typing import Any, cast, Sequence, Tuple, Mapping

import requests
from jsonschema.exceptions import ValidationError

from . import schemas, types
from ..pruning_validator import validate
from ..structure import structure_or_none, structure_list

MAIN_SERVER = 'https://naming.moera.org/moera-naming'
"""Main Moera naming server."""
DEV_SERVER = 'https://naming-dev.moera.org/moera-naming'
"""Moera developers' naming server."""


class MoeraNamingError(Exception):
    """Generic naming server error."""

    def __init__(self, method: str, message: str):
        """
        :param method: API method name
        :param message: error message
        """
        super().__init__(method + ': Naming server error: ' + message)


class MoeraNamingApiError(MoeraNamingError):
    """Naming server returned an error response."""
    error_code: int
    """Error code."""

    def __init__(self, name: str, response: Mapping[str, Any]):
        """
        :param method: API method name
        :param response: server response
        """
        super().__init__(name, response.get('message') if response.get('message') is not None else '')
        self.error_code = response.get('code', -1)


class MoeraNamingConnectionError(Exception):
    """Naming server connection error."""

    def __init__(self, message: str):
        """
        :param message: error message
        """
        super().__init__('Naming server connection error: ' + message)


CONNECTION_TIMEOUT = 10 # seconds

class MoeraNaming:
    """Naming API interface."""
    _server: str
    _call_id: int

    def __init__(self, server: str = MAIN_SERVER):
        """
        :param server: the naming server URL
        """
        self._server = server
        self._call_id = 0

    def call(self, method: str, params: Sequence[Any], schema: Any = None):
        """
        Generic naming API call.

        :param method: name of the method to be called
        :param params: parameters of the call
        :param schema: JSON schema to validate the return value
        :return: the return value, if any
        """
        try:
            r = requests.post(
                self._server,
                json={
                    'method': method,
                    'params': params,
                    'jsonrpc': '2.0',
                    'id': self._call_id,
                },
                timeout=CONNECTION_TIMEOUT
            )
            self._call_id += 1

            response = r.json()
            if r.status_code not in [200, 201] or 'error' in response:
                if 'error' in response:
                    raise MoeraNamingApiError(method, response['error'])
                else:
                    raise MoeraNamingError(method, 'Invalid server response: ' + repr(response))
            if 'result' not in response:
                raise MoeraNamingError(method, 'Invalid server response: ' + repr(response))
            result = response['result']
            if schema is not None and result is not None:
                validate(result, schema=schema)

            return result
        except requests.exceptions.InvalidJSONError as e:
            raise MoeraNamingError(method, 'Invalid server response') from e
        except requests.exceptions.RequestException as e:
            raise MoeraNamingConnectionError(str(e)) from e
        except ValidationError as e:
            raise MoeraNamingError(method, 'Invalid server response: ' + repr(e)) from e

    def put(self, name: str, generation: int, updating_key: bytes | None = None, node_uri: str | None = None,
            signing_key: bytes | None = None, valid_from: types.Timestamp | None = None,
            previous_digest: bytes | None = None, signature: bytes | None = None) -> str:
        """
        Register or update the name. See Architecture Overview for the `detailed description
        <https://moera.org/overview/naming.html>`_ of the algorithm.

        :param name: the name to be registered/updated
        :param generation: the name generation to be registered/updated
        :param updating_key: the public key for verifying signatures of further updates of the name. May be ``None``
            if the current generation of the name is updated – the current key is preserved in this case.
        :param node_uri: URI of the REST API endpoint of the node to which the name is assigned. May be ``None`` -
            the current URI is preserved in this case.
        :param signing_key: the public key of the name owner. May be ``None`` – the current key is preserved in this
            case.
        :param valid_from: the moment in time the owner's key is valid from. May be ``None`` if ``signing_key`` is also
            ``None``.
        :param previous_digest: the unique identifier as reported by a naming server of the current state of the name.
            Used to detect the situations when the name was changed by someone else between sending the request and
            processing it. May be ``None`` if the name was never registered before.
        :param signature: the signature, if required, ``None`` otherwise
        :return: identifier of the operation that was created
        """
        return cast(str, self.call('put', [name, generation, _encode_bytes(updating_key), node_uri,
                                           _encode_bytes(signing_key), valid_from, _encode_bytes(previous_digest),
                                           _encode_bytes(signature)]))

    def get_status(self, operation_id: str) -> types.OperationStatusInfo | None:
        """
        Get the current status of the operation.

        :param operation_id:
        :return: the operation status or ``None``, if the operation ID is unknown
        """
        return structure_or_none(self.call('getStatus', [operation_id], schemas.OPERATION_STATUS_INFO_SCHEMA),
                                 types.OperationStatusInfo)

    def get_current(self, name: str, generation: int) -> types.RegisteredNameInfo | None:
        """
        Get current information about the given generation of the name.

        :param name:
        :param generation:
        :return: the information or ``None``, if the name/generation is not found
        """
        return structure_or_none(self.call('getCurrent', [name, generation], schemas.REGISTERED_NAME_INFO_SCHEMA),
                                 types.RegisteredNameInfo)

    def get_past(self, name: str, generation: int, at: types.Timestamp) -> types.RegisteredNameInfo | None:
        """
        Get past information about the given generation of the name.

        :param name:
        :param generation:
        :param at: the moment in time the information is related to
        :return: the information or ``None``, if the name/generation did not exist at the given moment
        """
        return structure_or_none(self.call('getPast', [name, generation, at], schemas.REGISTERED_NAME_INFO_SCHEMA),
                                 types.RegisteredNameInfo)

    def is_free(self, name: str, generation: int) -> bool:
        """
        Check if the given name is available for registration.

        :param name:
        :param generation:
        :return: ``True``, if the name is free, ``False`` otherwise
        """
        return cast(bool, self.call('isFree', [name, generation]))

    def get_similar(self, name: str) -> types.RegisteredNameInfo | None:
        """
        Find a name that is close to the given name.

        :param name:
        :return: information about the name or ``None``, if no name found that is close enough
        """
        return structure_or_none(self.call('getSimilar', [name], schemas.REGISTERED_NAME_INFO_SCHEMA),
                                 types.RegisteredNameInfo)

    def get_all_keys(self, name: str, generation: int) -> list[types.SigningKeyInfo]:
        """
        Get the whole history of signing keys for the given name.

        :param name:
        :param generation:
        :return: the keys
        """
        return structure_list(self.call('getAllKeys', [name, generation], schemas.SIGNING_KEY_INFO_ARRAY_SCHEMA),
                              types.SigningKeyInfo)

    def get_all(self, at: types.Timestamp, page: int, size: int) -> list[types.RegisteredNameInfo]:
        """
        Get the list of all registered names at the given moment. The list is returned in pages, one per call.

        :param at: the moment in time the information is related to
        :param page: number of the page to be returned (starting from 0)
        :param size: size of the page
        :return:
        """
        return structure_list(self.call('getAll', [at, page, size], schemas.REGISTERED_NAME_INFO_ARRAY_SCHEMA),
                              types.RegisteredNameInfo)

    def get_all_newer(self, at: types.Timestamp, page: int, size: int) -> list[types.RegisteredNameInfo]:
        """
        Get the list of all names registered after the given moment. The list is returned in pages, one per call.

        :param at: the moment in time the information is related to
        :param page: number of the page to be returned (starting from 0)
        :param size: size of the page
        :return:
        """
        return structure_list(self.call('getAllNewer', [at, page, size], schemas.REGISTERED_NAME_INFO_ARRAY_SCHEMA),
                              types.RegisteredNameInfo)


def _encode_bytes(b: bytes | None) -> str | None:
    return b64encode(b).decode() if b is not None else None


def node_name_parse(node_name: str) -> Tuple[str, int]:
    """
    Parse a node name and return its name and generation parts.

    If the node name does not include a generation, generation 0 is returned. If name syntax is invalid, ``ValueError``
    is raised.

    :param node_name: the node name to be parsed
    :return: tuple (name, generation)
    """
    name = node_name
    generation = 0

    pos = node_name.rfind('_')
    if pos >= 0:
        (name, gen) = (node_name[0:pos], node_name[pos + 1:])
        try:
            generation = int(gen)
        except ValueError:
            raise ValueError(f'invalid generation: "{gen}"')

    return name, generation


def shorten(node_name: str | None) -> str | None:
    """
    Converts the node name to the compact form, omitting generation 0.

    :param node_name: the node name in compact or full form
    :return: the node name in the compact form
    """
    if node_name is None:
        return None
    (name, gen) = node_name_parse(node_name)
    if gen == 0:
        return name
    else:
        return node_name


def expand(node_name: str | None) -> str | None:
    """
    Converts the node name to the full form, containing generation.

    :param node_name: the node name in compact or full form
    :return: the node name in the full form
    """
    if node_name is None:
        return None
    (name, gen) = node_name_parse(node_name)
    return f'{name}_{gen}'


def resolve(name: str, naming_server: str = MAIN_SERVER) -> str | None:
    """
    Shortcut function to resolve a node name and get the node URI.

    :param name: the node name
    :param naming_server: a naming server to be used
    :return: the node URI, or ``None`` if the name does not exist
    """
    (name, gen) = node_name_parse(name)
    naming = MoeraNaming(naming_server)
    info = naming.get_current(name, gen)
    return info.node_uri if info is not None else None



# This file is generated

from typing import List

from ..crypto.crypto import fingerprint_bytes
from ..crypto.fingerprint import FingerprintSchema
from .types import Timestamp

ATTACHMENT_FINGERPRINT0_SCHEMA: FingerprintSchema = [
    ('version', 'number'),
    ('object_type', 'string'),
    ('digest', 'bytes'),
]


def create_attachment_fingerprint0(digest: bytes | None) -> bytes:
    return fingerprint_bytes({'version': 0, 'object_type': 'ATTACHMENT'} | locals(), ATTACHMENT_FINGERPRINT0_SCHEMA)


CARTE_FINGERPRINT2_SCHEMA: FingerprintSchema = [
    ('version', 'number'),
    ('object_type', 'string'),
    ('owner_name', 'string'),
    ('address', 'string'),
    ('beginning', 'number'),
    ('deadline', 'number'),
    ('node_name', 'string'),
    ('client_scope', 'number'),
    ('admin_scope', 'number'),
    ('salt', 'bytes'),
]


def create_carte_fingerprint2(
    owner_name: str | None, address: str | None, beginning: Timestamp | None, deadline: Timestamp | None,
    node_name: str | None, client_scope: int | None, admin_scope: int | None, salt: bytes | None
) -> bytes:
    return fingerprint_bytes({'version': 2, 'object_type': 'CARTE'} | locals(), CARTE_FINGERPRINT2_SCHEMA)


CARTE_FINGERPRINT1_SCHEMA: FingerprintSchema = [
    ('version', 'number'),
    ('object_type', 'string'),
    ('owner_name', 'string'),
    ('address', 'string'),
    ('beginning', 'number'),
    ('deadline', 'number'),
    ('node_name', 'string'),
    ('auth_category', 'number'),
    ('salt', 'bytes'),
]


def create_carte_fingerprint1(
    owner_name: str | None, address: str | None, beginning: Timestamp | None, deadline: Timestamp | None,
    node_name: str | None, auth_category: int | None, salt: bytes | None
) -> bytes:
    return fingerprint_bytes({'version': 1, 'object_type': 'CARTE'} | locals(), CARTE_FINGERPRINT1_SCHEMA)


CARTE_FINGERPRINT0_SCHEMA: FingerprintSchema = [
    ('version', 'number'),
    ('object_type', 'string'),
    ('owner_name', 'string'),
    ('address', 'string'),
    ('beginning', 'number'),
    ('deadline', 'number'),
    ('permissions', 'number'),
    ('salt', 'bytes'),
]


def create_carte_fingerprint0(
    owner_name: str | None, address: str | None, beginning: Timestamp | None, deadline: Timestamp | None,
    permissions: int | None, salt: bytes | None
) -> bytes:
    return fingerprint_bytes({'version': 0, 'object_type': 'CARTE'} | locals(), CARTE_FINGERPRINT0_SCHEMA)


COMMENT_FINGERPRINT0_SCHEMA: FingerprintSchema = [
    ('version', 'number'),
    ('object_type', 'string'),
    ('owner_name', 'string'),
    ('posting_fingerprint', 'bytes'),
    ('replied_to_fingerprint', 'bytes'),
    ('body_src_hash', 'bytes'),
    ('body_src_format', 'string'),
    ('body', 'string'),
    ('body_format', 'string'),
    ('created_at', 'number'),
    ('permissions', 'number'),
    ('attachments', 'bytes[]'),
]


def create_comment_fingerprint0(
    owner_name: str | None, posting_fingerprint: bytes | None, replied_to_fingerprint: bytes | None,
    body_src_hash: bytes | None, body_src_format: str | None, body: str | None, body_format: str | None,
    created_at: Timestamp | None, permissions: int | None, attachments: List[bytes] | None
) -> bytes:
    return fingerprint_bytes({'version': 0, 'object_type': 'COMMENT'} | locals(), COMMENT_FINGERPRINT0_SCHEMA)


NOTIFICATION_PACKET_FINGERPRINT1_SCHEMA: FingerprintSchema = [
    ('version', 'number'),
    ('object_type', 'string'),
    ('id', 'string'),
    ('node_name', 'string'),
    ('full_name', 'string'),
    ('created_at', 'number'),
    ('type', 'string'),
    ('notification', 'string'),
]


def create_notification_packet_fingerprint1(
    id: str | None, node_name: str | None, full_name: str | None, created_at: Timestamp | None, type: str | None,
    notification: str | None
) -> bytes:
    return fingerprint_bytes(
        {'version': 1, 'object_type': 'NOTIFICATION_PACKET'} | locals(), NOTIFICATION_PACKET_FINGERPRINT1_SCHEMA
    )


NOTIFICATION_PACKET_FINGERPRINT0_SCHEMA: FingerprintSchema = [
    ('version', 'number'),
    ('object_type', 'string'),
    ('id', 'string'),
    ('node_name', 'string'),
    ('created_at', 'number'),
    ('type', 'string'),
    ('notification', 'string'),
]


def create_notification_packet_fingerprint0(
    id: str | None, node_name: str | None, created_at: Timestamp | None, type: str | None, notification: str | None
) -> bytes:
    return fingerprint_bytes(
        {'version': 0, 'object_type': 'NOTIFICATION_PACKET'} | locals(), NOTIFICATION_PACKET_FINGERPRINT0_SCHEMA
    )


POSTING_FINGERPRINT1_SCHEMA: FingerprintSchema = [
    ('version', 'number'),
    ('object_type', 'string'),
    ('receiver_name', 'string'),
    ('owner_name', 'string'),
    ('body_src_hash', 'bytes'),
    ('body_src_format', 'string'),
    ('body', 'string'),
    ('body_format', 'string'),
    ('created_at', 'number'),
    ('permissions', 'number'),
    ('attachments', 'bytes[]'),
]


def create_posting_fingerprint1(
    receiver_name: str | None, owner_name: str | None, body_src_hash: bytes | None, body_src_format: str | None,
    body: str | None, body_format: str | None, created_at: Timestamp | None, permissions: int | None,
    attachments: List[bytes] | None
) -> bytes:
    return fingerprint_bytes({'version': 1, 'object_type': 'POSTING'} | locals(), POSTING_FINGERPRINT1_SCHEMA)


POSTING_FINGERPRINT0_SCHEMA: FingerprintSchema = [
    ('version', 'number'),
    ('object_type', 'string'),
    ('receiver_name', 'string'),
    ('owner_name', 'string'),
    ('body_src_hash', 'bytes'),
    ('body_src_format', 'string'),
    ('body', 'string'),
    ('body_format', 'string'),
    ('created_at', 'number'),
    ('permissions', 'number'),
    ('attachments', 'number'),
]


def create_posting_fingerprint0(
    receiver_name: str | None, owner_name: str | None, body_src_hash: bytes | None, body_src_format: str | None,
    body: str | None, body_format: str | None, created_at: Timestamp | None, permissions: int | None,
    attachments: int | None
) -> bytes:
    return fingerprint_bytes({'version': 0, 'object_type': 'POSTING'} | locals(), POSTING_FINGERPRINT0_SCHEMA)


PUSH_RELAY_MESSAGE_FINGERPRINT0_SCHEMA: FingerprintSchema = [
    ('version', 'number'),
    ('object_type', 'string'),
    ('signed_at', 'number'),
]


def create_push_relay_message_fingerprint0(signed_at: Timestamp | None) -> bytes:
    return fingerprint_bytes(
        {'version': 0, 'object_type': 'PUSH_RELAY_MESSAGE'} | locals(), PUSH_RELAY_MESSAGE_FINGERPRINT0_SCHEMA
    )


PUSH_RELAY_REGISTER_FINGERPRINT0_SCHEMA: FingerprintSchema = [
    ('version', 'number'),
    ('object_type', 'string'),
    ('client_id', 'string'),
    ('lang', 'string'),
    ('signed_at', 'number'),
]


def create_push_relay_register_fingerprint0(
    client_id: str | None, lang: str | None, signed_at: Timestamp | None
) -> bytes:
    return fingerprint_bytes(
        {'version': 0, 'object_type': 'PUSH_RELAY_REGISTER'} | locals(), PUSH_RELAY_REGISTER_FINGERPRINT0_SCHEMA
    )


REACTION_FINGERPRINT0_SCHEMA: FingerprintSchema = [
    ('version', 'number'),
    ('object_type', 'string'),
    ('owner_name', 'string'),
    ('entry_fingerprint', 'bytes'),
    ('negative', 'boolean'),
    ('emoji', 'number'),
]


def create_reaction_fingerprint0(
    owner_name: str | None, entry_fingerprint: bytes | None, negative: bool | None, emoji: int | None
) -> bytes:
    return fingerprint_bytes({'version': 0, 'object_type': 'REACTION'} | locals(), REACTION_FINGERPRINT0_SCHEMA)


SHERIFF_ORDER_FINGERPRINT0_SCHEMA: FingerprintSchema = [
    ('version', 'number'),
    ('object_type', 'string'),
    ('sheriff_name', 'string'),
    ('node_name', 'string'),
    ('feed_name', 'string'),
    ('entry_fingerprint', 'bytes'),
    ('category', 'string'),
    ('reason_code', 'string'),
    ('reason_details', 'string'),
    ('created_at', 'number'),
]


def create_sheriff_order_fingerprint0(
    sheriff_name: str | None, node_name: str | None, feed_name: str | None, entry_fingerprint: bytes | None,
    category: str | None, reason_code: str | None, reason_details: str | None, created_at: Timestamp | None
) -> bytes:
    return fingerprint_bytes(
        {'version': 0, 'object_type': 'SHERIFF_ORDER'} | locals(), SHERIFF_ORDER_FINGERPRINT0_SCHEMA
    )



from __future__ import annotations

from base64 import b64decode
from typing import Any, TypeVar, get_type_hints, get_args, Mapping, List

from camel_converter import dict_to_snake, to_camel


class Structure:
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

    @classmethod
    def from_json(cls, data) -> Structure | None:
        if data is None:
            return None
        data = dict_to_snake(data)
        for (attr, value) in data.items():
            if value is not None:
                if is_bytes_attribute(cls, attr):
                    data[attr] = b64decode(value)
                else:
                    struct = get_structure_attribute_type(cls, attr)
                    if struct is not None and not isinstance(value, Structure):
                        if isinstance(value, list):
                            data[attr] = [struct.from_json(v) for v in value]
                        else:
                            data[attr] = struct.from_json(value)
        instance = cls()
        instance.__dict__.update(data)
        return instance

    @staticmethod
    def list_to_json(a_list):
        out = []
        for item in a_list:
            if isinstance(item, Structure):
                out.append(item.json())
            elif isinstance(item, list):
                out.append(Structure.list_to_json(item))
            else:
                out.append(item)
        return out

    def json(self):
        json = {}
        for name, value in self.__dict__.items():
            if name.startswith('_'):
                continue
            if isinstance(value, Structure):
                value = value.json()
            elif isinstance(value, list):
                value = Structure.list_to_json(value)
            json[to_camel(name)] = value
        return json

    def __repr__(self) -> str:
        return repr(self.json())


StructType = TypeVar('StructType', bound=Structure)


def is_bytes_attribute(struct_type: type[StructType], attr: str) -> bool:
    hints = get_type_hints(struct_type)
    return attr in hints and (hints[attr] == bytes or bytes in get_args(hints[attr]))


def is_structure(type_hint: Any) -> bool:
    return isinstance(type_hint, type) and issubclass(type_hint, Structure)


def is_list_generic(type_hint: Any) -> bool:
    return hasattr(type_hint, '__origin__') and type_hint.__origin__ == list


def get_structure_attribute_type(struct_type: type[StructType], attr: str) -> type[Structure] | None:
    hints = get_type_hints(struct_type)
    if attr not in hints:
        return None
    if is_structure(hints[attr]):
        return hints[attr]
    for hint in get_args(hints[attr]):
        if is_structure(hint):
            return hint
        if is_list_generic(hint):
            for list_hint in get_args(hint):
                if is_structure(list_hint):
                    return list_hint
    return None


def array_schema(schema: Any) -> Any:
    return {
        "type": "array",
        "items": schema
    }


def to_nullable_object_schema(schema: Any) -> Any:
    result = schema.copy()
    result.update(type=["object", "null"])
    return result


def structure_or_none(data, struct_type: type[StructType]) -> StructType | None:
    return struct_type.from_json(data) if data is not None else None


def structure_list(data, struct_type: type[StructType]) -> List[StructType]:
    return [struct_type.from_json(item) for item in data]


def comma_separated_flags(flags: Mapping[str, bool]) -> str | None:
    names = [name for name, value in flags.items() if value is not None]
    return ','.join(names) if len(names) > 0 else None



from typing import TypeAlias, Literal

from ..structure import Structure

Timestamp: TypeAlias = int
OperationStatus: TypeAlias = Literal['WAITING', 'ADDED', 'STARTED', 'SUCCEEDED', 'FAILED', 'UNKNOWN']


class OperationStatusInfo(Structure):
    operation_id: str
    name: str
    """the name that was created or updated by the operation"""
    generation: int
    """the generation of the name that was created or updated by the operation"""
    status: OperationStatus
    """the current status of the operation"""
    added: Timestamp | None = None
    """when the operation was added"""
    completed: Timestamp | None = None
    """when the operation was completed"""
    error_code: str | None = None
    """the error code, if the operation has been failed"""
    error_message: str | None = None
    """the human-readable error description, if the operation has been failed"""


class RegisteredNameInfo(Structure):
    name: str
    generation: int
    updating_key: bytes | None = None
    """the public key for verifying signatures of further updates of the name"""
    node_uri: str
    """URI of the REST API endpoint of the node to which the name is assigned"""
    created: Timestamp | None = None
    """when the name was created"""
    signing_key: bytes | None = None
    """the public key of the name owner. May be ``None``."""
    valid_from: Timestamp | None = None
    """the moment in time the owner's key is valid from. May be absent if ``signingKey`` is also absent."""
    digest: bytes | None = None
    """
    a unique identifier of the current state of the name. May be a transaction ID of the latest transaction with
    this name in the blockchain, or a cryptographic digest of the current state.
    """


class SigningKeyInfo(Structure):
    key: bytes
    """the public key"""
    valid_from: Timestamp
    """the moment in time the key is valid from"""
